# coding: utf8
from __future__ import unicode_literals

from ..char_classes import LIST_PUNCT, LIST_ELLIPSES, LIST_QUOTES
from ..char_classes import QUOTES, UNITS, ALPHA, ALPHA_LOWER, ALPHA_UPPER

LIST_ICONS = [r'[\p{So}--[°]]']

_currency = r'\$|¢|£|€|¥|฿'
_quotes = QUOTES.replace("'", '')

_prefixes = ([r'\+'] + LIST_PUNCT + LIST_ELLIPSES + LIST_QUOTES + LIST_ICONS +
             [r'[,.:](?=[{a}])'.format(a=ALPHA)])

_suffixes = (LIST_PUNCT + LIST_ELLIPSES + LIST_QUOTES + LIST_ICONS +
             [r'(?<=[0-9])\+',
              r'(?<=°[FfCcKk])\.',
              r'(?<=[0-9])(?:{})'.format(_currency),
              r'(?<=[0-9])(?:{})'.format(UNITS),
              r'(?<=[{}{}{}(?:{})])\.'.format(ALPHA_LOWER, r'%²\-\)\]\+', QUOTES, _currency),
              r'(?<=[{})])-e'.format(ALPHA_LOWER)])

_infixes = (LIST_ELLIPSES + LIST_ICONS +
            [r'(?<=[{}])\.(?=[{}])'.format(ALPHA_LOWER, ALPHA_UPPER),
             r'(?<=[{a}])[,!?](?=[{a}])'.format(a=ALPHA),
             r'(?<=[{a}"])[:<>=](?=[{a}])'.format(a=ALPHA),
             r'(?<=[{a}])--(?=[{a}])'.format(a=ALPHA),
             r'(?<=[{a}]),(?=[{a}])'.format(a=ALPHA),
             r'(?<=[{a}])([{q}\)\]\(\[])(?=[\-{a}])'.format(a=ALPHA, q=_quotes)])

TOKENIZER_PREFIXES = _prefixes
TOKENIZER_SUFFIXES = _suffixes
TOKENIZER_INFIXES = _infixes



# coding: utf8
from __future__ import unicode_literals

import plac
from pathlib import Path

from .converters import conllu2json, iob2json, conll_ner2json
from ..util import prints

# Converters are matched by file extension. To add a converter, add a new
# entry to this dict with the file extension mapped to the converter function
# imported from /converters.
CONVERTERS = {
    'conllu': conllu2json,
    'conll': conllu2json,
    'ner': conll_ner2json,
    'iob': iob2json,
}


@plac.annotations(
    input_file=("input file", "positional", None, str),
    output_dir=("output directory for converted file", "positional", None, str),
    n_sents=("Number of sentences per doc", "option", "n", int),
    converter=("Name of converter (auto, iob, conllu or ner)", "option", "c", str),
    morphology=("Enable appending morphology to tags", "flag", "m", bool))
def convert(input_file, output_dir, n_sents=1, morphology=False, converter='auto'):
    """
    Convert files into JSON format for use with train command and other
    experiment management functions.
    """
    input_path = Path(input_file)
    output_path = Path(output_dir)
    if not input_path.exists():
        prints(input_path, title="Input file not found", exits=1)
    if not output_path.exists():
        prints(output_path, title="Output directory not found", exits=1)
    if converter == 'auto':
        converter = input_path.suffix[1:]
    if converter not in CONVERTERS:
            prints("Can't find converter for %s" % converter,
                title="Unknown format", exits=1)
    func = CONVERTERS[converter]
    func(input_path, output_path,
         n_sents=n_sents, use_morphology=morphology)



# coding: utf8
from __future__ import unicode_literals

import plac
import platform
from pathlib import Path

from ..compat import path2str
from .. import about
from .. import util


@plac.annotations(
    model=("optional: shortcut link of model", "positional", None, str),
    markdown=("generate Markdown for GitHub issues", "flag", "md", str))
def info(model=None, markdown=False):
    """Print info about spaCy installation. If a model shortcut link is
    speficied as an argument, print model information. Flag --markdown
    prints details in Markdown for easy copy-pasting to GitHub issues.
    """
    if model:
        if util.is_package(model):
            model_path = util.get_package_path(model)
        else:
            model_path = util.get_data_path() / model
        meta_path = model_path / 'meta.json'
        if not meta_path.is_file():
            util.prints(meta_path, title="Can't find model meta.json", exits=1)
        meta = util.read_json(meta_path)
        if model_path.resolve() != model_path:
            meta['link'] = path2str(model_path)
            meta['source'] = path2str(model_path.resolve())
        else:
            meta['source'] = path2str(model_path)
        print_info(meta, 'model %s' % model, markdown)
    else:
        data = {'spaCy version': about.__version__,
                'Location': path2str(Path(__file__).parent.parent),
                'Platform': platform.platform(),
                'Python version': platform.python_version(),
                'Models': list_models()}
        print_info(data, 'spaCy', markdown)


def print_info(data, title, markdown):
    title = 'Info about %s' % title
    if markdown:
        util.print_markdown(data, title=title)
    else:
        util.print_table(data, title=title)


def list_models():
    def exclude_dir(dir_name):
        # exclude common cache directories and hidden directories
        exclude = ['cache', 'pycache', '__pycache__']
        return dir_name in exclude or dir_name.startswith('.')
    data_path = util.get_data_path()
    if data_path:
        models = [f.parts[-1] for f in data_path.iterdir() if f.is_dir()]
        return ', '.join([m for m in models if not exclude_dir(m)])
    return '-'



# coding: utf8
from __future__ import unicode_literals


NOUNS_IRREG = {
    "aardwolves": ("aardwolf",),
    "abaci": ("abacus",),
    "aboideaux": ("aboideau",),
    "aboiteaux": ("aboiteau",),
    "abscissae": ("abscissa",),
    "acanthi": ("acanthus",),
    "acari": ("acarus",),
    "acciaccature": ("acciaccatura",),
    "acetabula": ("acetabulum",),
    "achaemenidae": ("achaemenid",),
    "achaemenides": ("achaemenid",),
    "acicula": ("aciculum",),
    "aciculae": ("acicula",),
    "acini": ("acinus",),
    "acre-feet": ("acre-foot",),
    "acromia": ("acromion",),
    "actiniae": ("actinia",),
    "actinozoa": ("actinozoan",),
    "addenda": ("addendum",),
    "adenocarcinomata": ("adenocarcinoma",),
    "adenomata": ("adenoma",),
    "adieux": ("adieu",),
    "adyta": ("adytum",),
    "aecia": ("aecium",),
    "aecidia": ("aecidium",),
    "aerobia": ("aerobium",),
    "agents-general": ("agent-general",),
    "aggiornamenti": ("aggiornamento",),
    "agnomina": ("agnomen",),
    "agones": ("agon",),
    "agorae": ("agora",),
    "agouties": ("agouti",),
    "aides-de-camp": ("aide-de-camp",),
    "aides-memoire": ("aide-memoire",),
    "aids-de-camp": ("aid-de-camp",),
    "alae": ("ala",),
    "alewives": ("alewife",),
    "alkalies": ("alkali",),
    "allodia": ("allodium",),
    "alluvia": ("alluvium",),
    "alodia": ("alodium",),
    "alto-relievos": ("alto-relievo", "alto-rilievo",),
    "altocumuli": ("altocumulus",),
    "altostrati": ("altostratus",),
    "alulae": ("alula",),
    "alumnae": ("alumna",),
    "alumni": ("alumnus",),
    "alveoli": ("alveolus",),
    "amanuenses": ("amanuensis",),
    "ambulacra": ("ambulacrum",),
    "amebae": ("ameba",),
    "amici_curiae": ("amicus_curiae",),
    "amnia": ("amnion",),
    "amniocenteses": ("amniocentesis",),
    "amoebae": ("amoeba",),
    "amoebiases": ("amoebiasis",),
    "amoraim": ("amora",),
    "amoretti": ("amoretto",),
    "amorini": ("amorino",),
    "amphiarthroses": ("amphiarthrosis",),
    "amphicia": ("amphithecium",),
    "amphimixes": ("amphimixis",),
    "amphioxi": ("amphioxus",),
    "amphisbaenae": ("amphisbaena",),
    "amphorae": ("amphora",),
    "ampullae": ("ampulla",),
    "amygdalae": ("amygdala",),
    "anabases": ("anabasis",),
    "anacolutha": ("anacoluthon",),
    "anacruses": ("anacrusis",),
    "anaerobia": ("anaerobium",),
    "anagnorises": ("anagnorisis",),
    "analemmata": ("analemma",),
    "analyses": ("analysis",),
    "anamneses": ("anamnesis",),
    "anamorphoses": ("anamorphosis",),
    "anastomoses": ("anastomosis",),
    "anatyxes": ("anaptyxis",),
    "ancones": ("ancon", "ancone",),
    "androclinia": ("androclinium",),
    "androecia": ("androecium",),
    "androsphinges": ("androsphinx",),
    "andtheridia": ("antheridium",),
    "angelfishes": ("angelfish",),
    "angiomata": ("angioma",),
    "animalcula": ("animalculum",),
    "anlagen": ("anlage",),
    "annattos": ("anatto", "annatto",),
    "annuli": ("annulus",),
    "antae": ("anta",),
    "antalkalies": ("antalkali",),
    "antefixa": ("antefix",),
    "antennae": ("antenna",),
    "antependia": ("antependium",),
    "anthelia": ("anthelion",),
    "anthelices": ("anthelix",),
    "anthemia": ("anthemion",),
    "antheridia": ("antheridium",),
    "anthodia": ("anthodium",),
    "anthozoa": ("anthozoan",),
    "anthraces": ("anthrax",),
    "anticlinoria": ("anticlinorium",),
    "antihelices": ("antihelix",),
    "antiheroes": ("antihero",),
    "antisera": ("antiserum",),
    "antitheses": ("antithesis",),
    "antitragi": ("antitragus",),
    "antra": ("antrum",),
    "anus": ("anus",),
    "aortae": ("aorta",),
    "aphelia": ("aphelion",),
    "aphides": ("aphis",),
    "apices": ("apex",),
    "apodoses": ("apodosis",),
    "apomixes": ("apomixis",),
    "aponeuroses": ("aponeurosis",),
    "apophyses": ("apophysis",),
    "aposiopeses": ("aposiopesis",),
    "apothecia": ("apothecium",),
    "apotheoses": ("apotheosis",),
    "apparatus": ("apparatus",),
    "appendices": ("appendix",),
    "appoggiature": ("appoggiatura",),
    "apsides": ("apsis",),
    "aquae": ("aqua",),
    "aquaria": ("aquarium",),
    "araglis": ("argali",),
    "arboreta": ("arboretum",),
    "arcana": ("arcanum",),
    "archegonia": ("archegonium",),
    "archerfishes": ("archerfish",),
    "archesporia": ("archesporium",),
    "archipelagoes": ("archipelago",),
    "arcs-boutants": ("arc-boutant",),
    "areolae": ("areola",),
    "argali": ("argali",),
    "argumenta": ("argumentum",),
    "ariette": ("arietta",),
    "aristae": ("arista",),
    "armamentaria": ("armamentarium",),
    "arses": ("arsis",),
    "artal": ("rotl",),
    "artel": ("rotl",),
    "arterioscleroses": ("arteriosclerosis",),
    "aruspices": ("aruspex",),
    "asceses": ("ascesis",),
    "asci": ("ascus",),
    "ascidia": ("ascidium",),
    "ascogonia": ("ascogonium",),
    "ashes": ("ash",),
    "ashkenazim": ("ashkenazi",),
    "aspergilla": ("aspergillum",),
    "aspergilli": ("aspergillus",),
    "aspergilloses": ("aspergillosis",),
    "aspersoria": ("aspersorium",),
    "assegais": ("assagai", "assegai",),
    "astragali": ("astragalus",),
    "asyndeta": ("asyndeton",),
    "atheromata": ("atheroma",),
    "atheroscleroses": ("atherosclerosis",),
    "atmolyses": ("atmolysis",),
    "atria": ("atrium",),
    "attorneys-at-law": ("attorney-at-law",),
    "auditoria": ("auditorium",),
    "aurae": ("aura",),
    "aurar": ("eyrir",),
    "aurei": ("aureus",),
    "auriculae": ("auricula",),
    "aurorae": ("aurora",),
    "auspices": ("auspex", "auspice",),
    "autocatalyses": ("autocatalysis",),
    "autochthones": ("autochthon",),
    "automata": ("automaton",),
    "autos-da-fe": ("auto-da-fe",),
    "avitaminoses": ("avitaminosis",),
    "axes": ("ax", "axis",),
    "axillae": ("axilla",),
    "bacchantes": ("bacchant", "bacchante",),
    "bacchii": ("bacchius",),
    "bacilli": ("bacillus",),
    "bacteriostases": ("bacteriostasis",),
    "bacula": ("baculum",),
    "bains-marie": ("bain-marie",),
    "bains_marie": ("bain_marie",),
    "ballistae": ("ballista",),
    "bambini": ("bambino",),
    "bandeaux": ("bandeau",),
    "banditti": ("bandit",),
    "bani": ("ban",),
    "banjoes": ("banjo",),
    "barklice": ("barklouse",),
    "barramundies": ("barramundi",),
    "bases": ("base", "basis",),
    "bases-on-balls": ("base_on_balls",),
    "bases_on_balls": ("base_on_balls",),
    "basidia": ("basidium",),
    "basileis": ("basileus",),
    "bassi": ("basso",),
    "bastinadoes": ("bastinado",),
    "bateaux": ("bateau",),
    "batfishes": ("batfish",),
    "beadsmen": ("beadsman", "bedesman",),
    "beaux": ("beau",),
    "beches-de-mer": ("beche-de-mer",),
    "beeves": ("beef",),
    "behooves": ("behoof",),
    "bersaglieri": ("bersagliere",),
    "bhishties": ("bheesty", "bhishti",),
    "bibliothecae": ("bibliotheca",),
    "bicennaries": ("bicentenary", "bicentennial",),
    "bijoux": ("bijou",),
    "bilboes": ("bilbo",),
    "billets-doux": ("billet-doux",),
    "billfishes": ("billfish",),
    "bimboes": ("bimbo",),
    "bisectrices": ("bisectrix",),
    "blackfeet": ("blackfoot",),
    "blackfishes": ("blackfish",),
    "blastemata": ("blastema",),
    "blastulae": ("blastula",),
    "blindfishes": ("blindfish",),
    "blowfishes": ("blowfish",),
    "bluefishes": ("bluefish",),
    "boarfishes": ("boarfish",),
    "bok": ("boschbok",),
    "boleti": ("boletus",),
    "bolivares": ("bolivar",),
    "bolsheviki": ("bolshevik",),
    "bonefishes": ("bonefish",),
    "bongoes": ("bongo",),
    "bonitoes": ("bonito",),
    "booklice": ("booklouse",),
    "bookshelves": ("bookshelf",),
    "boraces": ("borax",),
    "borborygmi": ("borborygmus",),
    "bordereaux": ("bordereau",),
    "botargoes": ("botargo",),
    "box-kodaks": ("box_kodak",),
    "boxfishes": ("boxfish",),
    "brachia": ("brachium",),
    "brainchildren": ("brainchild",),
    "branchiae": ("branchia",),
    "brants": ("brant", "brent",),
    "bravadoes": ("bravado",),
    "bravoes": ("bravo",),
    "bregmata": ("bregma",),
    "brethren": ("brother",),
    "broadcast_media": ("broadcast_medium",),
    "broadleaves": ("broadleaf",),
    "bronchi": ("bronchus",),
    "brothers-in-law": ("brother-in-law",),
    "bryozoa": ("bryozoan",),
    "buboes": ("bubo",),
    "buckoes": ("bucko",),
    "buckteeth": ("bucktooth",),
    "buffaloes": ("buffalo",),
    "bullae": ("bulla",),
    "bunde": ("bund",),
    "bureaux": ("bureau",),
    "bureaux_de_change": ("bureau_de_change",),
    "bursae": ("bursa",),
    "bushbok": ("boschbok",),
    "bushboks": ("boschbok",),
    "busses": ("bus",),
    "butterfishes": ("butterfish",),
    "byssi": ("byssus",),
    "cacti": ("cactus",),
    "caducei": ("caduceus",),
    "caeca": ("caecum",),
    "caesurae": ("caesura",),
    "calami": ("calamus",),
    "calathi": ("calathus",),
    "calcanei": ("calcaneum", "calcaneus",),
    "calces": ("calx",),
    "calculi": ("calculus",),
    "caldaria": ("caldarium",),
    "calices": ("calix",),
    "calicoes": ("calico",),
    "calli": ("callus",),
    "calves": ("calf",),
    "calyces": ("calyx",),
    "cambia": ("cambium",),
    "camerae": ("camera",),
    "canaliculi": ("canaliculus",),
    "candelabra": ("candelabrum",),
    "candlefishes": ("candlefish",),
    "canthi": ("canthus",),
    "canulae": ("canula",),
    "canzoni": ("canzone",),
    "capita": ("caput",),
    "capitula": ("capitulum",),
    "capricci": ("capriccio",),
    "carabinieri": ("carabiniere",),
    "carbonadoes": ("carbonado",),
    "carcinomata": ("carcinoma",),
    "cargoes": ("cargo",),
    "carides": ("caryatid",),
    "carinae": ("carina",),
    "caroli": ("carolus",),
    "carpi": ("carpus",),
    "carpogonia": ("carpogonium",),
    "carryings-on": ("carrying-on",),
    "caryopses": ("caryopsis",),
    "caryopsides": ("caryopsis",),
    "castrati": ("castrato",),
    "catabases": ("catabasis",),
    "cataclases": ("cataclasis",),
    "cataloes": ("catalo",),
    "catalyses": ("catalysis",),
    "catenae": ("catena",),
    "catfishes": ("catfish",),
    "cathari": ("cathar",),
    "cathexes": ("cathexis",),
    "cattaloes": ("cattalo",),
    "caudices": ("caudex",),
    "caules": ("caulis",),
    "cavatine": ("cavatina",),
    "cavefishes": ("cavefish",),
    "cavetti": ("cavetto",),
    "cavo-rilievi": ("cavo-rilievo",),
    "ceca": ("cecum",),
    "cellae": ("cella",),
    "cembali": ("cembalo",),
    "centesimi": ("centesimo",),
    "centra": ("centrum",),
    "cephalothoraces": ("cephalothorax",),
    "cercariae": ("cercaria",),
    "cercariiae": ("cercaria",),
    "cerci": ("cercus",),
    "cerebella": ("cerebellum",),
    "cerebra": ("cerebrum",),
    "cervices": ("cervix",),
    "cestuses": ("caestus",),
    "cesurae": ("cesura",),
    "chadarim": ("cheder",),
    "chaetae": ("chaeta",),
    "chaises_longues": ("chaise_longue",),
    "chalazae": ("chalaza",),
    "challoth": ("hallah",),
    "chalutzim": ("chalutz",),
    "chapaties": ("chapati",),
    "chapatties": ("chapatti",),
    "chapeaux": ("chapeau",),
    "chasidim": ("chasid",),
    "chassidim": ("chassid",),
    "chateaux": ("chateau",),
    "chazanim": ("chazan",),
    "chedarim": ("cheder",),
    "chefs-d'ouvre": ("chef-d'ouvre",),
    "chelae": ("chela",),
    "chelicerae": ("chelicera",),
    "cherubim": ("cherub",),
    "chevaux-de-frise": ("cheval-de-frise",),
    "chiasmata": ("chiasma",),
    "chiasmi": ("chiasmus",),
    "children": ("child",),
    "chillies": ("chilli",),
    "chinese_eddoes": ("chinese_eddo",),
    "chitarroni": ("chitarrone",),
    "chlamydes": ("chlamys",),
    "chlamyses": ("chlamys",),
    "chondromata": ("chondroma",),
    "choragi": ("choragus",),
    "choriambi": ("choriambus",),
    "choux": ("chou",),
    "chromonemata": ("chromonema",),
    "chrysalides": ("chrysalis",),
    "chuvashes": ("chuvash",),
    "ciboria": ("ciborium",),
    "cicadae": ("cicada",),
    "cicale": ("cicala",),
    "cicatrices": ("cicatrix",),
    "ciceroni": ("cicerone",),
    "cicisbei": ("cicisbeo",),
    "cilia": ("cilium",),
    "cimices": ("cimex",),
    "cineraria": ("cinerarium",),
    "cingula": ("cingulum",),
    "cirri": ("cirrus",),
    "cirrocumuli": ("cirrocumulus",),
    "cirrostrati": ("cirrostratus",),
    "ciscoes": ("cisco",),
    "cisternae": ("cisterna",),
    "clani": ("clarino",),
    "clanos": ("clarino",),
    "claroes": ("claro",),
    "clepsydrae": ("clepsydra",),
    "clinandria": ("clinandrium",),
    "clingfishes": ("clingfish",),
    "clitella": ("clitellum",),
    "cloacae": ("cloaca",),
    "clostridia": ("clostridium",),
    "cloverleaves": ("cloverleaf",),
    "clypei": ("clypeus",),
    "coagula": ("coagulum",),
    "coalfishes": ("coalfish",),
    "cocci": ("coccus",),
    "coccyges": ("coccyx",),
    "cochleae": ("cochlea",),
    "codfishes": ("codfish",),
    "codices": ("codex",),
    "coelentera": ("coelenteron",),
    "coenuri": ("coenurus",),
    "cognomina": ("cognomen",),
    "cognosenti": ("cognosente",),
    "cola": ("colon",),
    "coleorhizae": ("coleorhiza",),
    "collegia": ("collegium",),
    "colloquia": ("colloquium",),
    "colluvia": ("colluvium",),
    "collyria": ("collyrium",),
    "colones": ("colon",),
    "colossi": ("colossus",),
    "columbaria": ("columbarium",),
    "columellae": ("columella",),
    "comae": ("coma",),
    "comatulae": ("comatula",),
    "comedones": ("comedo",),
    "comics": ("comic_strip", "comic",),
    "commandoes": ("commando",),
    "concertanti": ("concertante",),
    "concerti": ("concerto",),
    "concerti_grossi": ("concerto_grosso",),
    "concertini": ("concertino",),
    "conchae": ("concha",),
    "condottieri": ("condottiere",),
    "condylomata": ("condyloma",),
    "confervae": ("conferva",),
    "congii": ("congius",),
    "conidia": ("conidium",),
    "conjunctivae": ("conjunctiva",),
    "conquistadores": ("conquistador",),
    "consortia": ("consortium",),
    "contagia": ("contagium",),
    "continua": ("continuum",),
    "contralti": ("contralto",),
    "conversazioni": ("conversazione",),
    "convolvuli": ("convolvulus",),
    "cooks-general": ("cook-general",),
    "copulae": ("copula",),
    "corbiculae": ("corbicula",),
    "coria": ("corium",),
    "corneae": ("cornea",),
    "cornua": ("cornu",),
    "coronae": ("corona",),
    "corpora": ("corpus",),
    "corpora_lutea": ("corpus_luteum",),
    "corpora_striata": ("corpus_striatum",),
    "corrigenda": ("corrigendum",),
    "cortices": ("cortex",),
    "cortinae": ("cortina",),
    "corybantes": ("corybant",),
    "coryphaei": ("coryphaeus",),
    "costae": ("costa",),
    "cothurni": ("cothurnus",),
    "courts_martial": ("court_martial",),
    "couteaux": ("couteau",),
    "cowfishes": ("cowfish",),
    "coxae": ("coxa",),
    "cramboes": ("crambo",),
    "crania": ("cranium",),
    "crases": ("crasis",),
    "crawfishes": ("crawfish",),
    "crayfishes": ("crayfish",),
    "credenda": ("credendum",),
    "crematoria": ("crematorium",),
    "crescendi": ("crescendo",),
    "cribella": ("cribellum",),
    "crises": ("crisis",),
    "crissa": ("crissum",),
    "cristae": ("crista",),
    "criteria": ("criterion",),
    "cruces": ("crux",),
    "crura": ("crus",),
    "crusadoes": ("crusado",),
    "cruzadoes": ("cruzado",),
    "crying": ("cry",),
    "cryings": ("cry",),
    "ctenidia": ("ctenidium",),
    "cubicula": ("cubiculum",),
    "culices": ("culex",),
    "culpae": ("culpa",),
    "culs-de-sac": ("cul-de-sac",),
    "culti": ("cultus",),
    "cumuli": ("cumulus",),
    "cumulonimbi": ("cumulonimbus",),
    "cumulostrati": ("cumulostratus",),
    "curiae": ("curia",),
    "curricula": ("curriculum",),
    "custodes": ("custos",),
    "cutes": ("cutis",),
    "cuticulae": ("cuticula",),
    "cuttlefishes": ("cuttlefish",),
    "cyclopes": ("cyclops",),
    "cycloses": ("cyclosis",),
    "cylices": ("cylix",),
    "cylikes": ("cylix",),
    "cymae": ("cyma",),
    "cymatia": ("cymatium",),
    "cypselae": ("cypsela",),
    "cysticerci": ("cysticercus",),
    "dadoes": ("dado",),
    "dagoes": ("dago",),
    "damselfishes": ("damselfish",),
    "data": ("datum",),
    "daughters-in-law": ("daughter-in-law",),
    "daymio": ("daimio",),
    "daymios": ("daimio",),
    "dealfishes": ("dealfish",),
    "decemviri": ("decemvir",),
    "decennia": ("decennium",),
    "deciduae": ("decidua",),
    "definienda": ("definiendum",),
    "definientia": ("definiens",),
    "delphinia": ("delphinium",),
    "denarii": ("denarius",),
    "dentalia": ("dentalium",),
    "dermatoses": ("dermatosis",),
    "desiderata": ("desideratum",),
    "desperadoes": ("desperado",),
    "devilfishes": ("devilfish",),
    "diaereses": ("diaeresis",),
    "diaerses": ("diaeresis",),
    "diagnoses": ("diagnosis",),
    "dialyses": ("dialysis",),
    "diaphyses": ("diaphysis",),
    "diapophyses": ("diapophysis",),
    "diarthroses": ("diarthrosis",),
    "diastalses": ("diastalsis",),
    "diastases": ("diastasis",),
    "diastemata": ("diastema",),
    "diathses": ("diathesis",),
    "diazoes": ("diazo",),
    "dibbukkim": ("dibbuk",),
    "dichasia": ("dichasium",),
    "dicta": ("dictum",),
    "didoes": ("dido",),
    "diereses": ("dieresis",),
    "dieses": ("diesis",),
    "differentiae": ("differentia",),
    "dilettanti": ("dilettante",),
    "diluvia": ("diluvium",),
    "dingoes": ("dingo",),
    "diplococci": ("diplococcus",),
    "directors-general": ("director-general",),
    "disci": ("discus",),
    "discoboli": ("discobolos", "discobolus",),
    "dive": ("diva",),
    "diverticula": ("diverticulum",),
    "divertimenti": ("divertimento",),
    "djinn": ("djinni", "djinny",),
    "dodoes": ("dodo",),
    "dogfishes": ("dogfish",),
    "dogmata": ("dogma",),
    "dogteeth": ("dogtooth",),
    "dollarfishes": ("dollarfish",),
    "domatia": ("domatium",),
    "dominoes": ("domino",),
    "dormice": ("dormouse",),
    "dorsa": ("dorsum",),
    "drachmae": ("drachma",),
    "drawknives": ("drawknife",),
    "drosophilae": ("drosophila",),
    "drumfishes": ("drumfish",),
    "dryades": ("dryad",),
    "dui": ("duo",),
    "duona": ("duodenum",),
    "duonas": ("duodenum",),
    "dupondii": ("dupondius",),
    "duumviri": ("duumvir",),
    "dwarves": ("dwarf",),
    "dybbukkim": ("dybbuk",),
    "ecchymoses": ("ecchymosis",),
    "ecclesiae": ("ecclesia",),
    "ecdyses": ("ecdysis",),
    "echidnae": ("echidna",),
    "echini": ("echinus",),
    "echinococci": ("echinococcus",),
    "echoes": ("echo",),
    "ectozoa": ("ectozoan",),
    "eddoes": ("eddo",),
    "edemata": ("edema",),
    "effluvia": ("effluvium",),
    "eidola": ("eidolon",),
    "eisegeses": ("eisegesis",),
    "eisteddfodau": ("eisteddfod",),
    "elenchi": ("elenchus",),
    "ellipses": ("ellipsis",),
    "eluvia": ("eluvium",),
    "elves": ("elf",),
    "elytra": ("elytron", "elytrum",),
    "embargoes": ("embargo",),
    "emboli": ("embolus",),
    "emphases": ("emphasis",),
    "emporia": ("emporium",),
    "enarthroses": ("enarthrosis",),
    "encephala": ("encephalon",),
    "encephalitides": ("encephalitis",),
    "encephalomata": ("encephaloma",),
    "enchiridia": ("enchiridion",),
    "enchondromata": ("enchondroma",),
    "encomia": ("encomium",),
    "endamebae": ("endameba",),
    "endamoebae": ("endamoeba",),
    "endocardia": ("endocardium",),
    "endocrania": ("endocranium",),
    "endometria": ("endometrium",),
    "endostea": ("endosteum",),
    "endostoses": ("endostosis",),
    "endothecia": ("endothecium",),
    "endothelia": ("endothelium",),
    "endotheliomata": ("endothelioma",),
    "endozoa": ("endozoan",),
    "enemata": ("enema",),
    "enneahedra": ("enneahedron",),
    "entamebae": ("entameba",),
    "entamoebae": ("entamoeba",),
    "entases": ("entasis",),
    "entera": ("enteron",),
    "entia": ("ens",),
    "entozoa": ("entozoan", "entozoon",),
    "epencephala": ("epencephalon",),
    "epentheses": ("epenthesis",),
    "epexegeses": ("epexegesis",),
    "ephemera": ("ephemeron",),
    "ephemerae": ("ephemera",),
    "ephemerides": ("ephemeris",),
    "ephori": ("ephor",),
    "epicalyces": ("epicalyx",),
    "epicanthi": ("epicanthus",),
    "epicardia": ("epicardium",),
    "epicedia": ("epicedium",),
    "epicleses": ("epiclesis",),
    "epididymides": ("epididymis",),
    "epigastria": ("epigastrium",),
    "epiglottides": ("epiglottis",),
    "epimysia": ("epimysium",),
    "epiphenomena": ("epiphenomenon",),
    "epiphyses": ("epiphysis",),
    "episterna": ("episternum",),
    "epithalamia": ("epithalamion", "epithalamium",),
    "epithelia": ("epithelium",),
    "epitheliomata": ("epithelioma",),
    "epizoa": ("epizoan", "epizoon",),
    "epyllia": ("epyllion",),
    "equilibria": ("equilibrium",),
    "equiseta": ("equisetum",),
    "eringoes": ("eringo",),
    "errata": ("erratum",),
    "eryngoes": ("eryngo",),
    "esophagi": ("esophagus",),
    "etyma": ("etymon",),
    "eucalypti": ("eucalyptus",),
    "eupatridae": ("eupatrid",),
    "euripi": ("euripus",),
    "exanthemata": ("exanthema",),
    "executrices": ("executrix",),
    "exegeses": ("exegesis",),
    "exempla": ("exemplum",),
    "exordia": ("exordium",),
    "exostoses": ("exostosis",),
    "extrema": ("extremum",),
    "eyeteeth": ("eyetooth",),
    "fabliaux": ("fabliau",),
    "faciae": ("facia",),
    "faculae": ("facula",),
    "faeroese": ("faeroese",),
    "fallfishes": ("fallfish",),
    "famuli": ("famulus",),
    "farmers-general": ("farmer-general",),
    "faroese": ("faroese",),
    "farragoes": ("farrago",),
    "fasciae": ("fascia",),
    "fasciculi": ("fasciculus",),
    "fathers-in-law": ("father-in-law",),
    "fatsoes": ("fatso",),
    "faunae": ("fauna",),
    "feculae": ("fecula",),
    "fedayeen": ("fedayee",),
    "feet": ("foot",),
    "fellaheen": ("fellah",),
    "fellahin": ("fellah",),
    "felones_de_se": ("felo_de_se",),
    "felos_de_se": ("felo_de_se",),
    "femora": ("femur",),
    "fenestellae": ("fenestella",),
    "fenestrae": ("fenestra",),
    "feriae": ("feria",),
    "fermate": ("fermata",),
    "ferulae": ("ferula",),
    "festschriften": ("festschrift",),
    "fetiales": ("fetial",),
    "fezzes": ("fez",),
    "fiascoes": ("fiasco",),
    "fibrillae": ("fibrilla",),
    "fibromata": ("fibroma",),
    "fibulae": ("fibula",),
    "ficoes": ("fico",),
    "fideicommissa": ("fideicommissum",),
    "fieldmice": ("fieldmouse",),
    "figs.": ("fig.",),
    "fila": ("filum",),
    "filariiae": ("filaria",),
    "filefishes": ("filefish",),
    "fimbriae": ("fimbria",),
    "fishes": ("fish",),
    "fishwives": ("fishwife",),
    "fistulae": ("fistula",),
    "flabella": ("flabellum",),
    "flagella": ("flagellum",),
    "flagstaves": ("flagstaff",),
    "flambeaux": ("flambeau",),
    "flamines": ("flamen",),
    "flamingoes": ("flamingo",),
    "flatfeet": ("flatfoot",),
    "flatfishes": ("flatfish",),
    "fleurs-de-lis": ("fleur-de-lis",),
    "fleurs-de-lys": ("fleur-de-lys",),
    "flights_of_stairs": ("flight_of_stairs",),
    "flittermice": ("flittermouse",),
    "flocci": ("floccus",),
    "flocculi": ("flocculus",),
    "florae": ("flora",),
    "floreant.": ("floreat",),
    "florilegia": ("florilegium",),
    "flowers-de-luce": ("flower-de-luce",),
    "flyleaves": ("flyleaf",),
    "foci": ("focus",),
    "folia": ("folium",),
    "fora": ("forum",),
    "foramina": ("foramen",),
    "forceps": ("forceps",),
    "forefeet": ("forefoot",),
    "foreteeth": ("foretooth",),
    "formicaria": ("formicarium",),
    "formulae": ("formula",),
    "fornices": ("fornix",),
    "fortes": ("fortis",),
    "fossae": ("fossa",),
    "foveae": ("fovea",),
    "foveolae": ("foveola",),
    "fractocumuli": ("fractocumulus",),
    "fractostrati": ("fractostratus",),
    "fraena": ("fraenum",),
    "frauen": ("frau",),
    "frena": ("frenum",),
    "frenula": ("frenulum",),
    "frescoes": ("fresco",),
    "fricandeaux": ("fricandeau",),
    "fricandoes": ("fricando",),
    "frijoles": ("frijol",),
    "frogfishes": ("frogfish",),
    "frontes": ("frons",),
    "frusta": ("frustum",),
    "fuci": ("fucus",),
    "fulcra": ("fulcrum",),
    "fumatoria": ("fumatorium",),
    "fundi": ("fundus",),
    "fungi": ("fungus",),
    "funiculi": ("funiculus",),
    "furcula": ("furculum",),
    "furculae": ("furcula",),
    "furfures": ("furfur",),
    "galeae": ("galea",),
    "gambadoes": ("gambado",),
    "gametangia": ("gametangium",),
    "gametoecia": ("gametoecium",),
    "gammadia": ("gammadion",),
    "ganglia": ("ganglion",),
    "garfishes": ("garfish",),
    "gas": ("gas",),
    "gasses": ("gas",),
    "gastrulae": ("gastrula",),
    "gateaux": ("gateau",),
    "gazeboes": ("gazebo",),
    "geckoes": ("gecko",),
    "geese": ("goose",),
    "gelsemia": ("gelsemium",),
    "gemboks": ("gemsbok",),
    "gembucks": ("gemsbuck",),
    "gemeinschaften": ("gemeinschaft",),
    "gemmae": ("gemma",),
    "genera": ("genus",),
    "generatrices": ("generatrix",),
    "geneses": ("genesis",),
    "genii": ("genius",),
    "gentes": ("gens",),
    "gentlemen-at-arms": ("gentleman-at-arms",),
    "gentlemen-farmers": ("gentleman-farmer",),
    "genua": ("genu",),
    "genus": ("genus",),
    "germina": ("germen",),
    "gesellschaften": ("gesellschaft",),
    "gestalten": ("gestalt",),
    "ghettoes": ("ghetto",),
    "gingivae": ("gingiva",),
    "gingkoes": ("gingko",),
    "ginglymi": ("ginglymus",),
    "ginkgoes": ("ginkgo",),
    "gippoes": ("gippo",),
    "glabellae": ("glabella",),
    "gladioli": ("gladiolus",),
    "glandes": ("glans",),
    "gliomata": ("glioma",),
    "glissandi": ("glissando",),
    "globefishes": ("globefish",),
    "globigerinae": ("globigerina",),
    "glochidcia": ("glochidium",),
    "glochidia": ("glochidium",),
    "glomeruli": ("glomerulus",),
    "glossae": ("glossa",),
    "glottides": ("glottis",),
    "glutaei": ("glutaeus",),
    "glutei": ("gluteus",),
    "gnoses": ("gnosis",),
    "goatfishes": ("goatfish",),
    "goboes": ("gobo",),
    "godchildren": ("godchild",),
    "goes": ("go",),
    "goings-over": ("going-over",),
    "goldfishes": ("goldfish",),
    "gomphoses": ("gomphosis",),
    "gonia": ("gonion",),
    "gonidia": ("gonidium",),
    "gonococci": ("gonococcus",),
    "goodwives": ("goodwife",),
    "goosefishes": ("goosefish",),
    "gorgoneia": ("gorgoneion",),
    "gospopoda": ("gospodin",),
    "governors_general": ("governor_general",),
    "goyim": ("goy",),
    "gps": ("gps",),
    "grafen": ("graf",),
    "graffiti": ("graffito",),
    "grandchildren": ("grandchild",),
    "grants-in-aid": ("grant-in-aid",),
    "granulomata": ("granuloma",),
    "gravamina": ("gravamen",),
    "grig-gris": ("gris-gris",),
    "groszy": ("grosz",),
    "grottoes": ("grotto",),
    "guilder": ("guilde",),
    "guilders": ("guilde", "guilder",),
    "guitarfishes": ("guitarfish",),
    "gummata": ("gumma",),
    "gurnard": ("gurnar",),
    "gurnards": ("gurnar", "gurnard",),
    "guttae": ("gutta",),
    "gymnasia": ("gymnasium",),
    "gynaecea": ("gynaeceum",),
    "gynaecia": ("gynaecium",),
    "gynecea": ("gynecium",),
    "gynecia": ("gynecium",),
    "gynoecea": ("gynoecium",),
    "gynoecia": ("gynoecium",),
    "gyri": ("gyrus",),
    "hadarim": ("heder",),
    "hadjes": ("hadj",),
    "haematolyses": ("haematolysis",),
    "haematomata": ("haematoma",),
    "haematozoa": ("haematozoon",),
    "haemodialyses": ("haemodialysis",),
    "haemolyses": ("haemolysis",),
    "haemoptyses": ("haemoptysis",),
    "haeredes": ("haeres",),
    "haftaroth": ("haftarah",),
    "hagfishes": ("hagfish",),
    "haggadas": ("haggada", "haggadah",),
    "haggadoth": ("haggada",),
    "hajjes": ("hajj",),
    "haleru": ("haler",),
    "halfpence": ("halfpenny",),
    "hallot": ("hallah",),
    "halloth": ("hallah",),
    "halluces": ("hallux",),
    "haloes": ("halo",),
    "halteres": ("halter", "haltere",),
    "halves": ("half",),
    "hamuli": ("hamulus",),
    "hangers-on": ("hanger-on",),
    "haphtaroth": ("haphtarah",),
    "haredim": ("haredi",),
    "haruspices": ("haruspex",),
    "hasidim": ("hasid",),
    "hassidim": ("hassid",),
    "haustella": ("haustellum",),
    "haustoria": ("haustorium",),
    "hazzanim": ("hazzan",),
    "hectocotyli": ("hectocotylus",),
    "heirs-at-law": ("heir-at-law",),
    "heldentenore": ("heldentenor",),
    "helices": ("helix",),
    "heliozoa": ("heliozoan",),
    "hematolyses": ("hematolysis",),
    "hematomata": ("hematoma",),
    "hematozoa": ("hematozoon",),
    "hemelytra": ("hemelytron",),
    "hemielytra": ("hemielytron",),
    "hemodialyses": ("hemodialysis",),
    "hemolyses": ("hemolysis",),
    "hemoptyses": ("hemoptysis",),
    "hendecahedra": ("hendecahedron",),
    "hens-and-chickens": ("hen-and-chickens",),
    "heraclidae": ("heraclid",),
    "heraklidae": ("heraklid",),
    "herbaria": ("herbarium",),
    "hermae": ("herm", "herma",),
    "hermai": ("herma",),
    "herniae": ("hernia",),
    "heroes": ("hero",),
    "herren": ("herr",),
    "hetaerae": ("hetaera",),
    "hetairai": ("hetaira",),
    "hibernacula": ("hibernaculum",),
    "hieracosphinges": ("hieracosphinx",),
    "hila": ("hilum",),
    "hili": ("hilus",),
    "himatia": ("himation",),
    "hippocampi": ("hippocampus",),
    "hippopotami": ("hippopotamus",),
    "his": ("his",),
    "hoboes": ("hobo",),
    "hogfishes": ("hogfish",),
    "homunculi": ("homunculus",),
    "honoraria": ("honorarium",),
    "hooves": ("hoof",),
    "horologia": ("horologium",),
    "houses_of_cards": ("house_of_cards",),
    "housewives": ("housewife",),
    "humeri": ("humerus",),
    "hydrae": ("hydra",),
    "hydromedusae": ("hydromedusa",),
    "hydrozoa": ("hydrozoan",),
    "hymenoptera": ("hymenopteran",),
    "hynia": ("hymenium",),
    "hyniums": ("hymenium",),
    "hypanthia": ("hypanthium",),
    "hyperostoses": ("hyperostosis",),
    "hyphae": ("hypha",),
    "hypnoses": ("hypnosis",),
    "hypochondria": ("hypochondrium",),
    "hypogastria": ("hypogastrium",),
    "hypogea": ("hypogeum",),
    "hypophyses": ("hypophysis",),
    "hypostases": ("hypostasis",),
    "hypothalami": ("hypothalamus",),
    "hypotheses": ("hypothesis",),
    "hyraces": ("hyrax",),
    "iambi": ("iamb",),
    "ibices": ("ibex",),
    "ibo": ("igbo",),
    "ichthyosauri": ("ichthyosaurus",),
    "ichthyosauruses": ("ichthyosaur", "ichthyosaurus",),
    "iconostases": ("iconostas", "iconostasis",),
    "icosahedra": ("icosahedron",),
    "ideata": ("ideatum",),
    "igorrorote": ("igorrote",),
    "ilia": ("ilium",),
    "imagines": ("imago",),
    "imagoes": ("imago",),
    "imperia": ("imperium",),
    "impies": ("impi",),
    "incubi": ("incubus",),
    "incudes": ("incus",),
    "indices": ("index",),
    "indigoes": ("indigo",),
    "indumenta": ("indumentum",),
    "indusia": ("indusium",),
    "infundibula": ("infundibulum",),
    "ingushes": ("ingush",),
    "innuendoes": ("innuendo",),
    "inocula": ("inoculum",),
    "inquisitors-general": ("inquisitor-general",),
    "insectaria": ("insectarium",),
    "insulae": ("insula",),
    "intagli": ("intaglio",),
    "interleaves": ("interleaf",),
    "intermezzi": ("intermezzo",),
    "interreges": ("interrex",),
    "interregna": ("interregnum",),
    "intimae": ("intima",),
    "involucella": ("involucellum",),
    "involucra": ("involucrum",),
    "irides": ("iris",),
    "irs": ("irs",),
    "is": ("is",),
    "ischia": ("ischium",),
    "isthmi": ("isthmus",),
    "jackeroos": ("jackaroo", "jackeroo",),
    "jackfishes": ("jackfish",),
    "jackknives": ("jackknife",),
    "jacks-in-the-box": ("jack-in-the-box",),
    "jambeaux": ("jambeau",),
    "jellyfishes": ("jellyfish",),
    "jewelfishes": ("jewelfish",),
    "jewfishes": ("jewfish",),
    "jingoes": ("jingo",),
    "jinn": ("jinni",),
    "joes": ("jo", "joe",),
    "judge_advocates_general": ("judge_advocate_general",),
    "jura": ("jus",),
    "kaddishim": ("kaddish",),
    "kalmuck": ("kalmuc",),
    "kalmucks": ("kalmuc", "kalmuck",),
    "katabases": ("katabasis",),
    "keeshonden": ("keeshond",),
    "kibbutzim": ("kibbutz",),
    "killifishes": ("killifish",),
    "kingfishes": ("kingfish",),
    "kings-of-arms": ("king-of-arms",),
    "knights_bachelor": ("knight_bachelor",),
    "knights_bachelors": ("knight_bachelor",),
    "knights_templar": ("knight_templar",),
    "knights_templars": ("knight_templar",),
    "knives": ("knife",),
    "kohlrabies": ("kohlrabi",),
    "kronen": ("krone",),
    "kroner": ("krone",),
    "kronur": ("krona",),
    "krooni": ("kroon",),
    "kylikes": ("kylix",),
    "labara": ("labarum",),
    "labella": ("labellum",),
    "labia": ("labium",),
    "labra": ("labrum",),
    "lactobacilli": ("lactobacillus",),
    "lacunae": ("lacuna",),
    "lacunaria": ("lacunar",),
    "ladies-in-waiting": ("lady-in-waiting",),
    "lamellae": ("lamella",),
    "lamiae": ("lamia",),
    "laminae": ("lamina",),
    "lapilli": ("lapillus",),
    "lapithae": ("lapith",),
    "larvae": ("larva",),
    "larynges": ("larynx",),
    "lassoes": ("lasso",),
    "lati": ("lat",),
    "latices": ("latex",),
    "latifundia": ("latifundium",),
    "latu": ("lat",),
    "lavaboes": ("lavabo",),
    "leaves": ("leaf", "leave",),
    "lecythi": ("lecythus",),
    "leges": ("lex",),
    "lei": ("leu",),
    "lemmata": ("lemma",),
    "lemnisci": ("lemniscus",),
    "lenes": ("lenis",),
    "lentigines": ("lentigo",),
    "leonides": ("leonid",),
    "lepidoptera": ("lepidopteran",),
    "leprosaria": ("leprosarium",),
    "lepta": ("lepton",),
    "leptocephali": ("leptocephalus",),
    "leucocytozoa": ("leucocytozoan",),
    "leva": ("lev",),
    "librae": ("libra",),
    "libretti": ("libretto",),
    "lice": ("louse",),
    "lieder": ("lied",),
    "ligulae": ("ligula",),
    "limbi": ("limbus",),
    "limina": ("limen",),
    "limites": ("limes",),
    "limuli": ("limulus",),
    "lingoes": ("lingo",),
    "linguae": ("lingua",),
    "linguae_francae": ("lingua_franca",),
    "lionfishes": ("lionfish",),
    "lipomata": ("lipoma",),
    "lire": ("lira",),
    "liriodendra": ("liriodendron",),
    "lisente": ("sente",),
    "listente": ("sente",),
    "litai": ("lit", "litas",),
    "litu": ("litas",),
    "lives": ("life",),
    "lixivia": ("lixivium",),
    "loaves": ("loaf",),
    "loci": ("locus",),
    "loculi": ("loculus",),
    "loggie": ("loggia",),
    "logia": ("logion",),
    "lomenta": ("lomentum",),
    "longobardi": ("longobard",),
    "loricae": ("lorica",),
    "loups-garous": ("loup-garou",),
    "luba": ("luba",),
    "lubritoria": ("lubritorium",),
    "lumbus": ("lumbi",),
    "lumina": ("lumen",),
    "lumpfishes": ("lumpfish",),
    "lungfishes": ("lungfish",),
    "lunulae": ("lunula",),
    "lures": ("lur", "lure",),
    "lustra": ("lustre",),
    "lyings-in": ("lying-in",),
    "lymphangitides": ("lymphangitis",),
    "lymphomata": ("lymphoma",),
    "lymphopoieses": ("lymphopoiesis",),
    "lyses": ("lysis",),
    "lyttae": ("lytta",),
    "maare": ("maar",),
    "macaronies": ("macaroni",),
    "maccaronies": ("maccaroni",),
    "machzorim": ("machzor",),
    "macronuclei": ("macronucleus",),
    "macrosporangia": ("macrosporangium",),
    "maculae": ("macula",),
    "madornos": ("madrono",),
    "maestri": ("maestro",),
    "mafiosi": ("mafioso",),
    "magi": ("magus",),
    "magmata": ("magma",),
    "magnificoes": ("magnifico",),
    "mahzorim": ("mahzor",),
    "major-axes": ("major_axis",),
    "major_axes": ("major_axis",),
    "makuta": ("likuta",),
    "mallei": ("malleus",),
    "malleoli": ("malleolus",),
    "maloti": ("loti",),
    "mamillae": ("mamilla",),
    "mammae": ("mamma",),
    "mammillae": ("mammilla",),
    "mandingoes": ("mandingo",),
    "mangoes": ("mango",),
    "manifestoes": ("manifesto",),
    "manteaux": ("manteau",),
    "mantes": ("mantis",),
    "manubria": ("manubrium",),
    "marchese": ("marchesa",),
    "marchesi": ("marchese",),
    "maremme": ("maremma",),
    "markkaa": ("markka",),
    "marsupia": ("marsupium",),
    "marvels-of-peru": ("marvel-of-peru",),
    "mass_media": ("mass_medium",),
    "masses": ("mass", "masse",),
    "masters-at-arms": ("master-at-arms",),
    "matrices": ("matrix",),
    "matzoth": ("matzo",),
    "mausolea": ("mausoleum",),
    "maxillae": ("maxilla",),
    "maxima": ("maximum",),
    "media": ("medium",),
    "mediae": ("media",),
    "mediastina": ("mediastinum",),
    "medullae": ("medulla",),
    "medullae_oblongatae": ("medulla_oblongata",),
    "medusae": ("medusa",),
    "megara": ("megaron",),
    "megasporangia": ("megasporangium",),
    "megilloth": ("megillah",),
    "meioses": ("meiosis",),
    "melanomata": ("melanoma",),
    "melismata": ("melisma",),
    "mementoes": ("memento",),
    "memoranda": ("memorandum",),
    "men": ("man",),
    "men-at-arms": ("man-at-arms",),
    "men-o'-war": ("man-of-war",),
    "men-of-war": ("man-of-war",),
    "men_of_letters": ("man_of_letters",),
    "menisci": ("meniscus",),
    "menservants": ("manservant",),
    "menstrua": ("menstruum",),
    "mesdames": ("madame",),
    "mesdemoiselles": ("mademoiselle",),
    "mesentera": ("mesenteron",),
    "mesothoraces": ("mesothorax",),
    "messeigneurs": ("monseigneur",),
    "messieurs": ("monsieur",),
    "mestizoes": ("mestizo",),
    "metacarpi": ("metacarpus",),
    "metamorphoses": ("metamorphosis",),
    "metanephroi": ("metanephros",),
    "metastases": ("metastasis",),
    "metatarsi": ("metatarsus",),
    "metatheses": ("metathesis",),
    "metathoraces": ("metathorax",),
    "metazoa": ("metazoan",),
    "metempsychoses": ("metempsychosis",),
    "metencephala": ("metencephalon",),
    "mezuzoth": ("mezuzah",),
    "miasmata": ("miasma",),
    "mice": ("mouse",),
    "microanalyses": ("microanalysis",),
    "micrococci": ("micrococcus",),
    "micronuclei": ("micronucleus",),
    "microsporangia": ("microsporangium",),
    "midrashim": ("midrash",),
    "midwives": ("midwife",),
    "milia": ("milium",),
    "milieux": ("milieu",),
    "militated_against": ("militate_against",),
    "milkfishes": ("milkfish",),
    "millennia": ("millennium",),
    "minae": ("mina",),
    "minima": ("minimum",),
    "ministeria": ("ministerium",),
    "minutiae": ("minutia",),
    "minyanim": ("minyan",),
    "mioses": ("miosis",),
    "miracidia": ("miracidium",),
    "miri": ("mir",),
    "mishnayoth": ("mishna", "mishnah",),
    "mitochondria": ("mitochondrion",),
    "mitzvoth": ("mitzvah",),
    "modioli": ("modiolus",),
    "moduli": ("modulus",),
    "momenta": ("momentum",),
    "moments_of_truth": ("moment_of_truth",),
    "momi": ("momus",),
    "monades": ("monad", "monas",),
    "monkfishes": ("monkfish",),
    "monochasia": ("monochasium",),
    "monopodia": ("monopodium",),
    "monoptera": ("monopteron",),
    "monopteroi": ("monopteros",),
    "monsignori": ("monsignor",),
    "monts-de-piete": ("mont-de-piete",),
    "mooncalves": ("mooncalf",),
    "moonfishes": ("moonfish",),
    "morae": ("mora",),
    "moratoria": ("moratorium",),
    "morceaux": ("morceau",),
    "morescoes": ("moresco",),
    "moriscoes": ("morisco",),
    "morphallaxes": ("morphallaxis",),
    "morphoses": ("morphosis",),
    "morses": ("morse", "mors",),
    "morulae": ("morula",),
    "mosasauri": ("mosasaurus",),
    "moshavim": ("moshav",),
    "moslim": ("moslem",),
    "moslims": ("moslem",),
    "mosquitoes": ("mosquito",),
    "mothers-in-law": ("mother-in-law",),
    "mothers_superior": ("mother_superior",),
    "mottoes": ("motto",),
    "movers_and_shakers": ("mover_and_shaker",),
    "mucosae": ("mucosa",),
    "mucrones": ("mucro",),
    "mudejares": ("mudejar",),
    "mudfishes": ("mudfish",),
    "mulattoes": ("mulatto",),
    "multiparae": ("multipara",),
    "murices": ("murex",),
    "muskallunge": ("muskellunge",),
    "mycelia": ("mycelium",),
    "mycetomata": ("mycetoma",),
    "mycobacteria": ("mycobacterium",),
    "mycorrhizae": ("mycorrhiza",),
    "myelencephala": ("myelencephalon",),
    "myiases": ("myiasis",),
    "myocardia": ("myocardium",),
    "myofibrillae": ("myofibrilla",),
    "myomata": ("myoma",),
    "myoses": ("myosis",),
    "myrmidones": ("myrmidon",),
    "mythoi": ("mythos",),
    "myxomata": ("myxoma",),
    "naevi": ("naevus",),
    "naiades": ("naiad",),
    "naoi": ("naos",),
    "narcissi": ("narcissus",),
    "nares": ("naris",),
    "nasopharynges": ("nasopharynx",),
    "natatoria": ("natatorium",),
    "naumachiae": ("naumachia",),
    "nauplii": ("nauplius",),
    "nautili": ("nautilus",),
    "navahoes": ("navaho",),
    "navajoes": ("navajo",),
    "nebulae": ("nebula",),
    "necropoleis": ("necropolis",),
    "needlefishes": ("needlefish",),
    "negrilloes": ("negrillo",),
    "negritoes": ("negrito",),
    "negroes": ("negro",),
    "nemeses": ("nemesis",),
    "nephridia": ("nephridium",),
    "nereides": ("nereid",),
    "neurohypophyses": ("neurohypophysis",),
    "neuromata": ("neuroma",),
    "neuroptera": ("neuropteron",),
    "neuroses": ("neurosis",),
    "nevi": ("nevus",),
    "nibelungen": ("nibelung",),
    "nidi": ("nidus",),
    "nielli": ("niello",),
    "nilgai": ("nilgai",),
    "nimbi": ("nimbus",),
    "nimbostrati": ("nimbostratus",),
    "noctilucae": ("noctiluca",),
    "nodi": ("nodus",),
    "noes": ("no",),
    "nomina": ("nomen",),
    "nota": ("notum",),
    "noumena": ("noumenon",),
    "novae": ("nova",),
    "novelle": ("novella",),
    "novenae": ("novena",),
    "nubeculae": ("nubecula",),
    "nucelli": ("nucellus",),
    "nuchae": ("nucha",),
    "nuclei": ("nucleus",),
    "nucleoli": ("nucleolus",),
    "nulliparae": ("nullipara",),
    "numbfishes": ("numbfish",),
    "numina": ("numen",),
    "nymphae": ("nympha",),
    "oarfishes": ("oarfish",),
    "oases": ("oasis",),
    "obeli": ("obelus",),
    "objets_d'art": ("objet_d'art",),
    "obligati": ("obligato",),
    "oboli": ("obolus",),
    "occipita": ("occiput",),
    "oceanaria": ("oceanarium",),
    "oceanides": ("oceanid",),
    "ocelli": ("ocellus",),
    "ochreae": ("ochrea",),
    "ocreae": ("ochrea", "ocrea",),
    "octahedra": ("octahedron",),
    "octopi": ("octopus",),
    "oculi": ("oculus",),
    "odea": ("odeum",),
    "oedemata": ("edema", "oedema",),
    "oesophagi": ("esophagus", "oesophagus",),
    "oldwives": ("oldwife",),
    "olea": ("oleum",),
    "omasa": ("omasum",),
    "omayyades": ("omayyad",),
    "omenta": ("omentum",),
    "ommatidia": ("ommatidium",),
    "ommiades": ("ommiad",),
    "onagri": ("onager",),
    "oogonia": ("oogonium",),
    "oothecae": ("ootheca",),
    "operas_seria": ("opera_seria",),
    "opercula": ("operculum",),
    "optic_axes": ("optic_axis",),
    "optima": ("optimum",),
    "ora": ("os",),
    "organa": ("organon", "organum",),
    "organums": ("organa", "organum",),
    "orthoptera": ("orthopteron",),
    "osar": ("os",),
    "oscula": ("osculum",),
    "ossa": ("os",),
    "osteomata": ("osteoma",),
    "ostia": ("ostium",),
    "ottomans": ("othman", "ottoman",),
    "ova": ("ovum",),
    "ovoli": ("ovolo",),
    "ovotestes": ("ovotestis",),
    "oxen": ("ox",),
    "oxymora": ("oxymoron",),
    "paddlefishes": ("paddlefish",),
    "paise": ("paisa",),
    "paleae": ("palea",),
    "palestrae": ("palestra",),
    "palingeneses": ("palingenesis",),
    "pallia": ("pallium",),
    "palmettoes": ("palmetto",),
    "palpi": ("palpus",),
    "pancratia": ("pancratium",),
    "panettoni": ("panettone",),
    "paparazzi": ("paparazzo",),
    "paperknives": ("paperknife",),
    "papillae": ("papilla",),
    "papillomata": ("papilloma",),
    "pappi": ("pappus",),
    "papulae": ("papula",),
    "papyri": ("papyrus",),
    "parabases": ("parabasis",),
    "paraleipses": ("paraleipsis", "paralipsis",),
    "paralyses": ("paralysis",),
    "paramecia": ("paramecium",),
    "paramenta": ("parament",),
    "paraphyses": ("paraphysis",),
    "parapodia": ("parapodium",),
    "parapraxes": ("parapraxis",),
    "paraselenae": ("paraselene",),
    "parashoth": ("parashah",),
    "parasyntheta": ("parasyntheton",),
    "parazoa": ("parazoan",),
    "parentheses": ("parenthesis",),
    "parerga": ("parergon",),
    "parhelia": ("parhelion",),
    "parietes": ("paries",),
    "paris-mutuels": ("pari-mutuel",),
    "parrotfishes": ("parrotfish",),
    "parulides": ("parulis",),
    "pasos_dobles": ("paso_doble",),
    "passers-by": ("passer-by",),
    "pastorali": ("pastorale",),
    "patagia": ("patagium",),
    "patellae": ("patella",),
    "patinae": ("patina",),
    "patresfamilias": ("paterfamilias",),
    "pease": ("pea",),
    "peccadilloes": ("peccadillo",),
    "pectines": ("pecten",),
    "pedaloes": ("pedalo",),
    "pedes": ("pes",),
    "pekingese": ("pekinese",),
    "pelves": ("pelvis",),
    "pence": ("penny",),
    "penes": ("penis",),
    "penetralium": ("penetralia",),
    "penicillia": ("penicillium",),
    "penknives": ("penknife",),
    "pennae": ("penna",),
    "pennia": ("penni",),
    "pentahedra": ("pentahedron",),
    "pentimenti": ("pentimento",),
    "penumbrae": ("penumbra",),
    "pepla": ("peplum",),
    "pericardia": ("pericardium",),
    "perichondria": ("perichondrium",),
    "pericrania": ("pericranium",),
    "peridia": ("peridium",),
    "perigonia": ("perigonium",),
    "perihelia": ("perihelion",),
    "perinea": ("perineum",),
    "perinephria": ("perinephrium",),
    "perionychia": ("perionychium",),
    "periostea": ("periosteum",),
    "periphrases": ("periphrasis",),
    "peristalses": ("peristalsis",),
    "perithecia": ("perithecium",),
    "peritonea": ("peritoneum",),
    "personae": ("persona",),
    "petechiae": ("petechia",),
    "pfennige": ("pfennig",),
    "phalanges": ("phalange", "phalanx",),
    "phalli": ("phallus",),
    "pharynges": ("pharynx",),
    "phenomena": ("phenomenon",),
    "phi-phenomena": ("phi-phenomenon",),
    "philodendra": ("philodendron",),
    "phlyctenae": ("phlyctaena", "phlyctena",),
    "phyla": ("phylum",),
    "phylae": ("phyle",),
    "phyllotaxes": ("phyllotaxis",),
    "phylloxerae": ("phylloxera",),
    "phylogeneses": ("phylogenesis",),
    "pieds-a-terre": ("pied-a-terre",),
    "pigfishes": ("pigfish",),
    "pilea": ("pileum",),
    "pilei": ("pileus",),
    "pineta": ("pinetum",),
    "pinfishes": ("pinfish",),
    "pinkoes": ("pinko",),
    "pinnae": ("pinna",),
    "pinnulae": ("pinnula",),
    "pipefishes": ("pipefish",),
    "pirogi": ("pirog",),
    "piscinae": ("piscina",),
    "pithecanthropi": ("pithecanthropus",),
    "pithoi": ("pithos",),
    "placeboes": ("placebo",),
    "placentae": ("placenta",),
    "planetaria": ("planetarium",),
    "planulae": ("planula",),
    "plasmodesmata": ("plasmodesma",),
    "plasmodia": ("plasmodium",),
    "plateaux": ("plateau",),
    "plectra": ("plectron", "plectrum",),
    "plena": ("plenum",),
    "pleura": ("pleuron",),
    "pleurae": ("pleura",),
    "plicae": ("plica",),
    "ploughmen": ("ploughman", "plowman",),
    "pneumobacilli": ("pneumobacillus",),
    "pneumococci": ("pneumococcus",),
    "pocketknives": ("pocketknife",),
    "podetia": ("podetium",),
    "podia": ("podium",),
    "poleis": ("polis",),
    "pollices": ("pollex",),
    "pollinia": ("pollinium",),
    "polychasia": ("polychasium",),
    "polyhedra": ("polyhedron",),
    "polyparia": ("polyparium",),
    "polypi": ("polypus",),
    "polyzoa": ("polyzoan",),
    "polyzoaria": ("polyzoarium",),
    "pontes": ("pons",),
    "pontifices": ("pontifex",),
    "portamenti": ("portamento",),
    "porticoes": ("portico",),
    "portmanteaux": ("portmanteau",),
    "postliminia": ("postliminium",),
    "potatoes": ("potato",),
    "praenomina": ("praenomen",),
    "praxes": ("praxis",),
    "predelle": ("predella",),
    "premaxillae": ("premaxilla",),
    "prenomina": ("prenomen",),
    "prese": ("presa",),
    "primi": ("primo",),
    "primigravidae": ("primigravida",),
    "primiparae": ("primipara",),
    "primordia": ("primordium",),
    "principia": ("principium",),
    "proboscides": ("proboscis",),
    "proces-verbaux": ("proces-verbal",),
    "proglottides": ("proglottid", "proglottis",),
    "prognoses": ("prognosis",),
    "prolegomena": ("prolegomenon",),
    "prolepses": ("prolepsis",),
    "promycelia": ("promycelium",),
    "pronephra": ("pronephros",),
    "pronephroi": ("pronephros",),
    "pronuclei": ("pronucleus",),
    "propositi": ("propositus",),
    "proptoses": ("proptosis",),
    "propyla": ("propylon",),
    "propylaea": ("propylaeum",),
    "proscenia": ("proscenium",),
    "prosencephala": ("prosencephalon",),
    "prostheses": ("prosthesis",),
    "prostomia": ("prostomium",),
    "protases": ("protasis",),
    "prothalamia": ("prothalamion", "prothalamium",),
    "prothalli": ("prothallus",),
    "prothallia": ("prothallium",),
    "prothoraces": ("prothorax",),
    "protonemata": ("protonema",),
    "protozoa": ("protozoan",),
    "proventriculi": ("proventriculus",),
    "provisoes": ("proviso",),
    "prytanea": ("prytaneum",),
    "psalteria": ("psalterium",),
    "pseudopodia": ("pseudopodium",),
    "psychoneuroses": ("psychoneurosis",),
    "psychoses": ("psychosis",),
    "pterygia": ("pterygium",),
    "pterylae": ("pteryla",),
    "ptoses": ("ptosis",),
    "pubes": ("pubis",),
    "pudenda": ("pudendum",),
    "puli": ("pul",),
    "pulvilli": ("pulvillus",),
    "pulvini": ("pulvinus",),
    "punchinelloes": ("punchinello",),
    "pupae": ("pupa",),
    "puparia": ("puparium",),
    "putamina": ("putamen",),
    "putti": ("putto",),
    "pycnidia": ("pycnidium",),
    "pygidia": ("pygidium",),
    "pylori": ("pylorus",),
    "pyxides": ("pyxis",),
    "pyxidia": ("pyxidium",),
    "qaddishim": ("qaddish",),
    "quadrennia": ("quadrennium",),
    "quadrigae": ("quadriga",),
    "qualia": ("quale",),
    "quanta": ("quantum",),
    "quarterstaves": ("quarterstaff",),
    "quezales": ("quezal",),
    "quinquennia": ("quinquennium",),
    "quizzes": ("quiz",),
    "rabatos": ("rabato", "rebato",),
    "rabbitfishes": ("rabbitfish",),
    "rachides": ("rhachis",),
    "radices": ("radix",),
    "radii": ("radius",),
    "radulae": ("radula",),
    "ramenta": ("ramentum",),
    "rami": ("ramus",),
    "ranulae": ("ranula",),
    "ranunculi": ("ranunculus",),
    "raphae": ("raphe",),
    "raphides": ("raphide", "raphis",),
    "ratfishes": ("ratfish",),
    "reales": ("real",),
    "rearmice": ("rearmouse",),
    "recta": ("rectum",),
    "recti": ("rectus",),
    "rectrices": ("rectrix",),
    "redfishes": ("redfish",),
    "rediae": ("redia",),
    "referenda": ("referendum",),
    "refugia": ("refugium",),
    "reguli": ("regulus",),
    "reis": ("real",),
    "relata": ("relatum",),
    "remiges": ("remex",),
    "reremice": ("rearmouse", "reremouse",),
    "reseaux": ("reseau",),
    "residua": ("residuum",),
    "responsa": ("responsum",),
    "retia": ("rete",),
    "retiarii": ("retiarius",),
    "reticula": ("reticulum",),
    "retinacula": ("retinaculum",),
    "retinae": ("retina",),
    "rhabdomyomata": ("rhabdomyoma",),
    "rhachides": ("rhachis",),
    "rhachises": ("rachis", "rhachis",),
    "rhinencephala": ("rhinencephalon",),
    "rhizobia": ("rhizobium",),
    "rhombi": ("rhombus",),
    "rhonchi": ("rhonchus",),
    "rhyta": ("rhyton",),
    "ribbonfishes": ("ribbonfish",),
    "ricercacari": ("ricercare",),
    "ricercari": ("ricercare",),
    "rickettsiae": ("rickettsia",),
    "rilievi": ("rilievo",),
    "rimae": ("rima",),
    "robes-de-chambre": ("robe-de-chambre",),
    "rockfishes": ("rockfish",),
    "roma": ("rom",),
    "romans-fleuves": ("roman-fleuve",),
    "rondeaux": ("rondeau",),
    "rosaria": ("rosarium",),
    "rosefishes": ("rosefish",),
    "rostella": ("rostellum",),
    "rostra": ("rostrum",),
    "rouleaux": ("rouleau",),
    "rugae": ("ruga",),
    "rumina": ("rumen",),
    "runners-up": ("runner-up",),
    "sacra": ("sacrum",),
    "sacraria": ("sacrarium",),
    "saguaros": ("saguaro", "sahuaro",),
    "sailfishes": ("sailfish",),
    "salespeople": ("salesperson",),
    "salmonellae": ("salmonella",),
    "salpae": ("salpa",),
    "salpinges": ("salpinx",),
    "saltarelli": ("saltarello",),
    "salvoes": ("salvo",),
    "sancta": ("sanctum",),
    "sanitaria": ("sanitarium",),
    "santimi": ("santims",),
    "saphenae": ("saphena",),
    "sarcophagi": ("sarcophagus",),
    "sartorii": ("sartorius",),
    "sassanidae": ("sassanid",),
    "sawfishes": ("sawfish",),
    "scaldfishes": ("scaldfish",),
    "scaleni": ("scalenus",),
    "scapulae": ("scapula",),
    "scarabaei": ("scarabaeus",),
    "scarves": ("scarf",),
    "schatchonim": ("schatchen", "shadchan",),
    "schemata": ("schema",),
    "scherzandi": ("scherzando",),
    "scherzi": ("scherzo",),
    "schmoes": ("schmo",),
    "scholia": ("scholium",),
    "schuln": ("schul",),
    "schutzstaffeln": ("schutzstaffel",),
    "scirrhi": ("scirrhus",),
    "scleromata": ("scleroma",),
    "scleroses": ("sclerosis",),
    "sclerotia": ("sclerotium",),
    "scoleces": ("scolex",),
    "scolices": ("scolex",),
    "scopulae": ("scopula",),
    "scoriae": ("scoria",),
    "scotomata": ("scotoma",),
    "scriptoria": ("scriptorium",),
    "scrota": ("scrotum",),
    "scudi": ("scudo",),
    "scuta": ("scutum",),
    "scutella": ("scutellum",),
    "scyphi": ("scyphus",),
    "scyphistomae": ("scyphistoma",),
    "scyphozoa": ("scyphozoan",),
    "secondi": ("secondo",),
    "secretaries-general": ("secretary-general",),
    "segni": ("segno",),
    "seleucidae": ("seleucid",),
    "selves": ("self",),
    "senores": ("senor",),
    "sensilla": ("sensillum",),
    "senti": ("sent",),
    "senussis": ("senusi", "senussi",),
    "separatrices": ("separatrix",),
    "sephardim": ("sephardi",),
    "septa": ("septum",),
    "septaria": ("septarium",),
    "septennia": ("septennium",),
    "sequelae": ("sequela",),
    "sequestra": ("sequestrum",),
    "sera": ("serum",),
    "seraphim": ("seraph",),
    "sestertia": ("sestertium",),
    "setae": ("seta",),
    "sgraffiti": ("sgraffito",),
    "shabbasim": ("shabbas",),
    "shabbatim": ("shabbat",),
    "shackoes": ("shacko",),
    "shadchanim": ("shadchan",),
    "shadchans": ("schatchen", "shadchan",),
    "shakoes": ("shako",),
    "shammosim": ("shammas", "shammes",),
    "sheatfishes": ("sheatfish",),
    "sheaves": ("sheaf",),
    "shellfishes": ("shellfish",),
    "shelves": ("shelf",),
    "shinleaves": ("shinleaf",),
    "shittim": ("shittah",),
    "shmoes": ("shmo",),
    "shofroth": ("shofar", "shophar",),
    "shophroth": ("shophar",),
    "shrewmice": ("shrewmouse",),
    "shuln": ("shul",),
    "siddurim": ("siddur",),
    "sigloi": ("siglos",),
    "signore": ("signora",),
    "signori": ("signior", "signore",),
    "signorine": ("signorina",),
    "siliquae": ("siliqua",),
    "silvae": ("silva",),
    "silverfishes": ("silverfish",),
    "simulacra": ("simulacrum",),
    "sincipita": ("sinciput",),
    "sinfonie": ("sinfonia",),
    "sisters-in-law": ("sister-in-law",),
    "sistra": ("sistrum",),
    "situlae": ("situla",),
    "smalti": ("smalto",),
    "snaggleteeth": ("snaggletooth",),
    "snailfishes": ("snailfish",),
    "snipefishes": ("snipefish",),
    "socmen": ("socman", "sokeman",),
    "sola": ("solum",),
    "solaria": ("solarium",),
    "solatia": ("solatium",),
    "soldi": ("soldo",),
    "soles": ("sol", "sole",),
    "solfeggi": ("solfeggio",),
    "soli": ("solo",),
    "solidi": ("solidus",),
    "somata": ("soma",),
    "sons-in-law": ("son-in-law",),
    "soprani": ("soprano",),
    "sordini": ("sordino",),
    "sori": ("sorus",),
    "soroses": ("sorosis",),
    "sovkhozy": ("sovkhoz",),
    "spadefishes": ("spadefish",),
    "spadices": ("spadix",),
    "spearfishes": ("spearfish",),
    "spectra": ("spectrum",),
    "specula": ("speculum",),
    "spermatia": ("spermatium",),
    "spermatogonia": ("spermatogonium",),
    "spermatozoa": ("spermatozoon",),
    "spermogonia": ("spermogonium",),
    "sphinges": ("sphinx",),
    "spicae": ("spica",),
    "spicula": ("spiculum",),
    "spirilla": ("spirillum",),
    "splayfeet": ("splayfoot",),
    "splenii": ("splenius",),
    "sporangia": ("sporangium",),
    "sporogonia": ("sporogonium",),
    "sporozoa": ("sporozoan",),
    "springhase": ("springhaas",),
    "spumoni": ("spumone",),
    "sputa": ("sputum",),
    "squamae": ("squama",),
    "squashes": ("squash",),
    "squillae": ("squilla",),
    "squirrelfishes": ("squirrelfish",),
    "squizzes": ("squiz",),
    "stadia": ("stadium",),
    "stamina": ("stamen",),
    "staminodia": ("staminodium",),
    "stapedes": ("stapes",),
    "staphylococci": ("staphylococcus",),
    "staretsy": ("starets",),
    "starfishes": ("starfish",),
    "startsy": ("starets",),
    "stelae": ("stele",),
    "stemmata": ("stemma",),
    "stenoses": ("stenosis",),
    "stepchildren": ("stepchild",),
    "sterna": ("sternum",),
    "stigmata": ("stigma",),
    "stimuli": ("stimulus",),
    "stipites": ("stipes",),
    "stirpes": ("stirps",),
    "stoae": ("stoa",),
    "stockfishes": ("stockfish",),
    "stomata": ("stoma",),
    "stomodaea": ("stomodaeum",),
    "stomodea": ("stomodeum",),
    "stonefishes": ("stonefish",),
    "stotinki": ("stotinka",),
    "stotkini": ("stotinka",),
    "strappadoes": ("strappado",),
    "strata": ("stratum",),
    "strati": ("stratus",),
    "stratocumuli": ("stratocumulus",),
    "street_children": ("street_child",),
    "streptococci": ("streptococcus",),
    "stretti": ("stretto",),
    "striae": ("stria",),
    "strobili": ("strobilus",),
    "stromata": ("stroma",),
    "strumae": ("struma",),
    "stuccoes": ("stucco",),
    "styli": ("stylus",),
    "stylopes": ("stylops",),
    "stylopodia": ("stylopodium",),
    "subcortices": ("subcortex",),
    "subdeliria": ("subdelirium",),
    "subgenera": ("subgenus",),
    "subindices": ("subindex",),
    "submucosae": ("submucosa",),
    "subphyla": ("subphylum",),
    "substrasta": ("substratum",),
    "succedanea": ("succedaneum",),
    "succubi": ("succubus",),
    "suckerfishes": ("suckerfish",),
    "suckfishes": ("suckfish",),
    "sudaria": ("sudarium",),
    "sudatoria": ("sudatorium",),
    "sulci": ("sulcus",),
    "summae": ("summa",),
    "sunfishes": ("sunfish",),
    "supercargoes": ("supercargo",),
    "superheroes": ("superhero",),
    "supernovae": ("supernova",),
    "superstrata": ("superstratum",),
    "surgeonfishes": ("surgeonfish",),
    "swamies": ("swami",),
    "sweetiewives": ("sweetiewife",),
    "swellfishes": ("swellfish",),
    "swordfishes": ("swordfish",),
    "syconia": ("syconium",),
    "syllabi": ("syllabus",),
    "syllepses": ("syllepsis",),
    "symphyses": ("symphysis",),
    "sympodia": ("sympodium",),
    "symposia": ("symposium",),
    "synapses": ("synapsis",),
    "synarthroses": ("synarthrosis",),
    "synclinoria": ("synclinorium",),
    "syncytia": ("syncytium",),
    "syndesmoses": ("syndesmosis",),
    "synopses": ("synopsis",),
    "syntagmata": ("syntagma",),
    "syntheses": ("synthesis",),
    "syphilomata": ("syphiloma",),
    "syringes": ("syrinx",),
    "syssarcoses": ("syssarcosis",),
    "tableaux": ("tableau",),
    "taeniae": ("taenia", "tenia",),
    "tali": ("talus",),
    "tallaisim": ("tallith",),
    "tallithes": ("tallith",),
    "tallitoth": ("tallith",),
    "tapeta": ("tapetum",),
    "tarantulae": ("tarantula",),
    "tarsi": ("tarsus",),
    "tarsometatarsi": ("tarsometatarsus",),
    "taxa": ("taxon",),
    "taxes": ("tax", "taxis",),
    "taxies": ("taxi",),
    "tectrices": ("tectrix",),
    "teeth": ("tooth",),
    "tegmina": ("tegmen",),
    "telae": ("tela",),
    "telamones": ("telamon",),
    "telangiectases": ("telangiectasia", "telangiectasis",),
    "telia": ("telium",),
    "tempi": ("tempo",),
    "tenacula": ("tenaculum",),
    "tenderfeet": ("tenderfoot",),
    "teniae": ("tenia",),
    "tenues": ("tenuis",),
    "teraphim": ("teraph",),
    "terata": ("teras",),
    "teredines": ("teredo",),
    "terga": ("tergum",),
    "termini": ("terminus",),
    "terraria": ("terrarium",),
    "terzetti": ("terzetto",),
    "tesserae": ("tessera",),
    "testae": ("testa",),
    "testes": ("testis",),
    "testudines": ("testudo",),
    "tetrahedra": ("tetrahedron",),
    "tetraskelia": ("tetraskelion",),
    "thalamencephala": ("thalamencephalon",),
    "thalami": ("thalamus",),
    "thalli": ("thallus",),
    "theatres-in-the-round": ("theatre-in-the-round",),
    "thecae": ("theca",),
    "therses": ("thyrse",),
    "thesauri": ("thesaurus",),
    "theses": ("thesis",),
    "thickleaves": ("thickleaf",),
    "thieves": ("thief",),
    "tholoi": ("tholos",),
    "thoraces": ("thorax",),
    "thrombi": ("thrombus",),
    "thymi": ("thymus",),
    "thyrsi": ("thyrsus",),
    "tibiae": ("tibia",),
    "tilefishes": ("tilefish",),
    "tintinnabula": ("tintinnabulum",),
    "titmice": ("titmouse",),
    "toadfishes": ("toadfish",),
    "tobaccoes": ("tobacco",),
    "tomatoes": ("tomato",),
    "tomenta": ("tomentum",),
    "tondi": ("tondo",),
    "tonneaux": ("tonneau",),
    "tophi": ("tophus",),
    "topoi": ("topos",),
    "tori": ("torus",),
    "tornadoes": ("tornado",),
    "torpedoes": ("torpedo",),
    "torsi": ("torso",),
    "touracos": ("touraco", "turaco",),
    "trabeculae": ("trabecula",),
    "tracheae": ("trachea",),
    "traditores": ("traditor",),
    "tragi": ("tragus",),
    "trapezia": ("trapezium",),
    "trapezohedra": ("trapezohedron",),
    "traumata": ("trauma",),
    "treponemata": ("treponema",),
    "trichinae": ("trichina",),
    "triclinia": ("triclinium",),
    "triennia": ("triennium",),
    "triforia": ("triforium",),
    "triggerfishes": ("triggerfish",),
    "trihedra": ("trihedron",),
    "triskelia": ("triskelion",),
    "trisoctahedra": ("trisoctahedron",),
    "triumviri": ("triumvir",),
    "trivia": ("trivium",),
    "trochleae": ("trochlea",),
    "tropaeola": ("tropaeolum",),
    "trous-de-loup": ("trou-de-loup",),
    "trousseaux": ("trousseau",),
    "trunkfishes": ("trunkfish",),
    "trymata": ("tryma",),
    "tubae": ("tuba",),
    "turves": ("turf",),
    "tympana": ("tympanum",),
    "tyros": ("tiro", "tyro",),
    "ubermenschen": ("ubermensch",),
    "uglies": ("ugli",),
    "uigurs": ("uighur",),
    "ulnae": ("ulna",),
    "ultimata": ("ultimatum",),
    "umbilici": ("umbilicus",),
    "umbones": ("umbo",),
    "umbrae": ("umbra",),
    "unci": ("uncus",),
    "uncidia": ("uredium",),
    "uredines": ("uredo",),
    "uredinia": ("uredinium",),
    "uredosori": ("uredosorus",),
    "urethrae": ("urethra",),
    "urinalyses": ("urinalysis",),
    "uteri": ("uterus",),
    "utriculi": ("utriculus",),
    "uvulae": ("uvula",),
    "vacua": ("vacuum",),
    "vagi": ("vagus", "vagus",),
    "vaginae": ("vagina",),
    "valleculae": ("vallecula",),
    "vaporetti": ("vaporetto",),
    "varices": ("varix",),
    "vasa": ("vas",),
    "vascula": ("vasculum",),
    "vela": ("velum",),
    "velamina": ("velamen",),
    "velaria": ("velarium",),
    "venae": ("vena",),
    "venae_cavae": ("vena_cava",),
    "ventriculi": ("ventriculus",),
    "vermes": ("vermis",),
    "verrucae": ("verruca",),
    "vertebrae": ("vertebra",),
    "vertices": ("vertex",),
    "vertigines": ("vertigo",),
    "vertigoes": ("vertigo",),
    "vesicae": ("vesica",),
    "vetoes": ("veto",),
    "vexilla": ("vexillum",),
    "viatica": ("viaticum",),
    "viatores": ("viator",),
    "vibracula": ("vibraculum",),
    "vibrissae": ("vibrissa",),
    "vice-chairman": ("vice-chairman",),
    "villi": ("villus",),
    "vimina": ("vimen",),
    "vincula": ("vinculum",),
    "viragoes": ("virago",),
    "vires": ("vis",),
    "virtuosi": ("virtuoso",),
    "vitae": ("vita",),
    "vitelli": ("vitellus",),
    "vittae": ("vitta",),
    "vivaria": ("vivarium",),
    "voces": ("vox",),
    "volcanoes": ("volcano",),
    "volkslieder": ("volkslied",),
    "volte": ("volta",),
    "volvae": ("volva",),
    "vorticellae": ("vorticella",),
    "vortices": ("vortex",),
    "vulvae": ("vulva",),
    "wagons-lits": ("wagon-lit",),
    "wahhabis": ("wahabi", "wahhabi",),
    "wanderjahre": ("wanderjahr",),
    "weakfishes": ("weakfish",),
    "werewolves": ("werewolf",),
    "wharves": ("wharf",),
    "whippers-in": ("whipper-in",),
    "whitefishes": ("whitefish",),
    "wives": ("wife",),
    "wolffishes": ("wolffish",),
    "wolves": ("wolf",),
    "woodlice": ("woodlouse",),
    "wreckfishes": ("wreckfish",),
    "wunderkinder": ("wunderkind",),
    "xiphisterna": ("xiphisternum",),
    "yeshivahs": ("yeshiva",),
    "yeshivoth": ("yeshiva",),
    "yogin": ("yogi",),
    "yourselves": ("yourself",),
    "zamindaris": ("zamindari", "zemindari",),
    "zecchini": ("zecchino",),
    "zeroes": ("zero",),
    "zoa": ("zoon",),
    "zoaeae": ("zoaea", "zoea",),
    "zoeae": ("zoea",),
    "zoeas": ("zoaea",),
    "zoonoses": ("zoonosis",),
    "zoosporangia": ("zoosporangium",)
}



# coding: utf8
from __future__ import unicode_literals

import pytest
import re

from ...lang.en import English
from ...tokenizer import Tokenizer


def test_issue1494():    
    infix_re = re.compile(r'''[^a-z]''')
    text_to_tokenize1 = 'token 123test'
    expected_tokens1 = ['token', '1', '2', '3', 'test']

    text_to_tokenize2 = 'token 1test'
    expected_tokens2 = ['token', '1test']

    text_to_tokenize3 = 'hello...test'
    expected_tokens3 = ['hello', '.', '.', '.', 'test']
    
    def my_tokenizer(nlp):
        return Tokenizer(nlp.vocab,
                     {},
                     infix_finditer=infix_re.finditer
    )

    nlp = English()

    nlp.tokenizer = my_tokenizer(nlp)

    tokenized_words1 = [token.text for token in nlp(text_to_tokenize1)]
    assert tokenized_words1 == expected_tokens1

    tokenized_words2 = [token.text for token in nlp(text_to_tokenize2)]
    assert tokenized_words2 == expected_tokens2

    tokenized_words3 = [token.text for token in nlp(text_to_tokenize3)]
    assert tokenized_words3 == expected_tokens3



# coding: utf8
from __future__ import unicode_literals

from ...symbols import LEMMA, PRON_LEMMA

# Source: Danish Universal Dependencies and http://fjern-uv.dk/pronom.php

# Note: The Danish Universal Dependencies specify Case=Acc for all instances
# of "den"/"det" even when the case is in fact "Nom". In the rules below, Case
# is left unspecified for "den" and "det".

MORPH_RULES = {
    "PRON": {
        "jeg":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Sing", "Case": "Nom", "Gender": "Com"},                      # Case=Nom|Gender=Com|Number=Sing|Person=1|PronType=Prs
        "mig":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Sing", "Case": "Acc", "Gender": "Com"},                      # Case=Acc|Gender=Com|Number=Sing|Person=1|PronType=Prs
        "min":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Sing", "Poss": "Yes", "Gender": "Com"},                      # Gender=Com|Number=Sing|Number[psor]=Sing|Person=1|Poss=Yes|PronType=Prs
        "mit":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Sing", "Poss": "Yes", "Gender": "Neut"},                     # Gender=Neut|Number=Sing|Number[psor]=Sing|Person=1|Poss=Yes|PronType=Prs
        "vor":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Sing", "Poss": "Yes", "Gender": "Com"},                      # Gender=Com|Number=Sing|Number[psor]=Plur|Person=1|Poss=Yes|PronType=Prs|Style=Form
        "vort":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Sing", "Poss": "Yes", "Gender": "Neut"},                     # Gender=Neut|Number=Sing|Number[psor]=Plur|Person=1|Poss=Yes|PronType=Prs|Style=Form
        "du":           {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Sing", "Case": "Nom", "Gender": "Com"},                      # Case=Nom|Gender=Com|Number=Sing|Person=2|PronType=Prs
        "dig":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Sing", "Case": "Acc", "Gender": "Com"},                      # Case=Acc|Gender=Com|Number=Sing|Person=2|PronType=Prs
        "din":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Sing", "Poss": "Yes", "Gender": "Com"},                      # Gender=Com|Number=Sing|Number[psor]=Sing|Person=2|Poss=Yes|PronType=Prs
        "dit":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Sing", "Poss": "Yes", "Gender": "Neut"},                     # Gender=Neut|Number=Sing|Number[psor]=Sing|Person=2|Poss=Yes|PronType=Prs
        "han":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Case": "Nom", "Gender": "Com"},                    # Case=Nom|Gender=Com|Number=Sing|Person=3|PronType=Prs
        "hun":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Case": "Nom", "Gender": "Com"},                    # Case=Nom|Gender=Com|Number=Sing|Person=3|PronType=Prs
        "den":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Gender": "Com"},                                   # Case=Acc|Gender=Com|Number=Sing|Person=3|PronType=Prs, See note above.
        "det":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Gender": "Neut"},                                  # Case=Acc|Gender=Neut|Number=Sing|Person=3|PronType=Prs See note above.
        "ham":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Case": "Acc", "Gender": "Com"},                    # Case=Acc|Gender=Com|Number=Sing|Person=3|PronType=Prs
        "hende":        {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Case": "Acc", "Gender": "Com"},                    # Case=Acc|Gender=Com|Number=Sing|Person=3|PronType=Prs
        "sin":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Poss": "Yes", "Gender": "Com", "Reflex": "Yes"},   # Gender=Com|Number=Sing|Number[psor]=Sing|Person=3|Poss=Yes|PronType=Prs|Reflex=Yes
        "sit":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Sing", "Poss": "Yes", "Gender": "Neut", "Reflex": "Yes"},  # Gender=Neut|Number=Sing|Number[psor]=Sing|Person=3|Poss=Yes|PronType=Prs|Reflex=Yes

        "vi":           {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Plur", "Case": "Nom", "Gender": "Com"},                      # Case=Nom|Gender=Com|Number=Plur|Person=1|PronType=Prs
        "os":           {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Plur", "Case": "Acc", "Gender": "Com"},                      # Case=Acc|Gender=Com|Number=Plur|Person=1|PronType=Prs
        "mine":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Plur", "Poss": "Yes"},                                       # Number=Plur|Number[psor]=Sing|Person=1|Poss=Yes|PronType=Prs
        "vore":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Number": "Plur", "Poss": "Yes"},                                       # Number=Plur|Number[psor]=Plur|Person=1|Poss=Yes|PronType=Prs|Style=Form
        "I":            {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Plur", "Case": "Nom", "Gender": "Com"},                      # Case=Nom|Gender=Com|Number=Plur|Person=2|PronType=Prs
        "jer":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Plur", "Case": "Acc", "Gender": "Com"},                      # Case=Acc|Gender=Com|Number=Plur|Person=2|PronType=Prs
        "dine":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Number": "Plur", "Poss": "Yes"},                                       # Number=Plur|Number[psor]=Sing|Person=2|Poss=Yes|PronType=Prs
        "de":           {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Plur", "Case": "Nom"},                                     # Case=Nom|Number=Plur|Person=3|PronType=Prs
        "dem":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Plur", "Case": "Acc"},                                     # Case=Acc|Number=Plur|Person=3|PronType=Prs
        "sine":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Number": "Plur", "Poss": "Yes", "Reflex": "Yes"},                    # Number=Plur|Number[psor]=Sing|Person=3|Poss=Yes|PronType=Prs|Reflex=Yes

        "vores":        {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "One", "Poss": "Yes"},                                                         # Number[psor]=Plur|Person=1|Poss=Yes|PronType=Prs
        "De":           {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Case": "Nom", "Gender": "Com"},                                        # Case=Nom|Gender=Com|Person=2|Polite=Form|PronType=Prs
        "Dem":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Case": "Acc", "Gender": "Com"},                                        # Case=Acc|Gender=Com|Person=2|Polite=Form|PronType=Prs
        "Deres":        {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Poss": "Yes"},                                                         # Person=2|Polite=Form|Poss=Yes|PronType=Prs
        "jeres":        {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Two", "Poss": "Yes"},                                                         # Number[psor]=Plur|Person=2|Poss=Yes|PronType=Prs
        "sig":          {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Case": "Acc", "Reflex": "Yes"},                                      # Case=Acc|Person=3|PronType=Prs|Reflex=Yes
        "hans":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Poss": "Yes"},                                                       # Number[psor]=Sing|Person=3|Poss=Yes|PronType=Prs
        "hendes":       {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Poss": "Yes"},                                                       # Number[psor]=Sing|Person=3|Poss=Yes|PronType=Prs
        "dens":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Poss": "Yes"},                                                       # Number[psor]=Sing|Person=3|Poss=Yes|PronType=Prs
        "dets":         {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Poss": "Yes"},                                                       # Number[psor]=Sing|Person=3|Poss=Yes|PronType=Prs
        "deres":        {LEMMA: PRON_LEMMA, "PronType": "Prs", "Person": "Three", "Poss": "Yes"},                                                       # Number[psor]=Plur|Person=3|Poss=Yes|PronType=Prs
    },

    "VERB": {
        "er":           {LEMMA: "være", "VerbForm": "Fin", "Tense": "Pres"},
        "var":          {LEMMA: "være", "VerbForm": "Fin", "Tense": "Past"}
    }
}

for tag, rules in MORPH_RULES.items():
    for key, attrs in dict(rules).items():
        rules[key.title()] = attrs



# coding: utf8
from __future__ import unicode_literals


# Source https://github.com/stopwords-iso/stopwords-fi/blob/master/stopwords-fi.txt
# Reformatted with some minor corrections

STOP_WORDS = set("""

aiemmin aika aikaa aikaan aikaisemmin aikaisin aikana aikoina aikoo aikovat
aina ainakaan ainakin ainoa ainoat aiomme aion aiotte aivan ajan alas alemmas
alkuisin alkuun alla alle aloitamme aloitan aloitat aloitatte aloitattivat
aloitettava aloitettavaksi aloitettu aloitimme aloitin aloitit aloititte
aloittaa aloittamatta aloitti aloittivat alta aluksi alussa alusta annettavaksi
annettava annettu ansiosta antaa antamatta antoi apu asia asiaa asian asiasta
asiat asioiden asioihin asioita asti avuksi avulla avun avutta

edelle edelleen edellä edeltä edemmäs edes edessä edestä ehkä ei eikä eilen
eivät eli ellei elleivät ellemme ellen ellet ellette emme en enemmän eniten
ennen ensi ensimmäinen ensimmäiseksi ensimmäisen ensimmäisenä ensimmäiset
ensimmäisiksi ensimmäisinä ensimmäisiä ensimmäistä ensin entinen entisen
entisiä entisten entistä enää eri erittäin erityisesti eräiden eräs eräät esi
esiin esillä esimerkiksi et eteen etenkin ette ettei että

halua haluaa haluamatta haluamme haluan haluat haluatte haluavat halunnut
halusi halusimme halusin halusit halusitte halusivat halutessa haluton he hei
heidän heidät heihin heille heillä heiltä heissä heistä heitä helposti heti
hetkellä hieman hitaasti huolimatta huomenna hyvien hyviin hyviksi hyville
hyviltä hyvin hyvinä hyvissä hyvistä hyviä hyvä hyvät hyvää hän häneen hänelle
hänellä häneltä hänen hänessä hänestä hänet häntä

ihan ilman ilmeisesti itse itsensä itseään

ja jo johon joiden joihin joiksi joilla joille joilta joina joissa joista joita
joka jokainen jokin joko joksi joku jolla jolle jolloin jolta jompikumpi jona
jonka jonkin jonne joo jopa jos joskus jossa josta jota jotain joten jotenkin
jotenkuten jotka jotta jouduimme jouduin jouduit jouduitte joudumme joudun
joudutte joukkoon joukossa joukosta joutua joutui joutuivat joutumaan joutuu
joutuvat juuri jälkeen jälleen jää

kahdeksan kahdeksannen kahdella kahdelle kahdelta kahden kahdessa kahdesta
kahta kahteen kai kaiken kaikille kaikilta kaikkea kaikki kaikkia kaikkiaan
kaikkialla kaikkialle kaikkialta kaikkien kaikkiin kaksi kannalta kannattaa
kanssa kanssaan kanssamme kanssani kanssanne kanssasi kauan kauemmas kaukana
kautta kehen keiden keihin keiksi keille keillä keiltä keinä keissä keistä
keitten keittä keitä keneen keneksi kenelle kenellä keneltä kenen kenenä
kenessä kenestä kenet kenettä kenties kerran kerta kertaa keskellä kesken
keskimäärin ketkä ketä kiitos kohti koko kokonaan kolmas kolme kolmen kolmesti
koska koskaan kovin kuin kuinka kuinkaan kuitenkaan kuitenkin kuka kukaan kukin
kumpainen kumpainenkaan kumpi kumpikaan kumpikin kun kuten kuuden kuusi kuutta
kylliksi kyllä kymmenen kyse

liian liki lisäksi lisää lla luo luona lähekkäin lähelle lähellä läheltä
lähemmäs lähes lähinnä lähtien läpi

mahdollisimman mahdollista me meidän meidät meihin meille meillä meiltä meissä
meistä meitä melkein melko menee menemme menen menet menette menevät meni
menimme menin menit menivät mennessä mennyt menossa mihin miksi mikä mikäli
mikään mille milloin milloinkan millä miltä minkä minne minua minulla minulle
minulta minun minussa minusta minut minuun minä missä mistä miten mitkä mitä
mitään moi molemmat mones monesti monet moni moniaalla moniaalle moniaalta
monta muassa muiden muita muka mukaan mukaansa mukana mutta muu muualla muualle
muualta muuanne muulloin muun muut muuta muutama muutaman muuten myöhemmin myös
myöskin myöskään myötä

ne neljä neljän neljää niiden niihin niiksi niille niillä niiltä niin niinä
niissä niistä niitä noiden noihin noiksi noilla noille noilta noin noina noissa
noista noita nopeammin nopeasti nopeiten nro nuo nyt näiden näihin näiksi
näille näillä näiltä näin näinä näissä näistä näitä nämä

ohi oikea oikealla oikein ole olemme olen olet olette oleva olevan olevat oli
olimme olin olisi olisimme olisin olisit olisitte olisivat olit olitte olivat
olla olleet ollut oma omaa omaan omaksi omalle omalta oman omassa omat omia
omien omiin omiksi omille omilta omissa omista on onkin onko ovat

paikoittain paitsi pakosti paljon paremmin parempi parhaillaan parhaiten
perusteella peräti pian pieneen pieneksi pienelle pienellä pieneltä pienempi
pienestä pieni pienin poikki puolesta puolestaan päälle

runsaasti

saakka sama samaa samaan samalla saman samat samoin sata sataa satojen se
seitsemän sekä sen seuraavat siellä sieltä siihen siinä siis siitä sijaan siksi
sille silloin sillä silti siltä sinne sinua sinulla sinulle sinulta sinun
sinussa sinusta sinut sinuun sinä sisäkkäin sisällä siten sitten sitä ssa sta
suoraan suuntaan suuren suuret suuri suuria suurin suurten

taa taas taemmas tahansa tai takaa takaisin takana takia tallä tapauksessa
tarpeeksi tavalla tavoitteena te teidän teidät teihin teille teillä teiltä
teissä teistä teitä tietysti todella toinen toisaalla toisaalle toisaalta
toiseen toiseksi toisella toiselle toiselta toisemme toisen toisensa toisessa
toisesta toista toistaiseksi toki tosin tuhannen tuhat tule tulee tulemme tulen
tulet tulette tulevat tulimme tulin tulisi tulisimme tulisin tulisit tulisitte
tulisivat tulit tulitte tulivat tulla tulleet tullut tuntuu tuo tuohon tuoksi
tuolla tuolle tuolloin tuolta tuon tuona tuonne tuossa tuosta tuota tuskin tykö
tähän täksi tälle tällä tällöin tältä tämä tämän tänne tänä tänään tässä tästä
täten tätä täysin täytyvät täytyy täällä täältä

ulkopuolella usea useasti useimmiten usein useita uudeksi uudelleen uuden uudet
uusi uusia uusien uusinta uuteen uutta

vaan vai vaiheessa vaikea vaikean vaikeat vaikeilla vaikeille vaikeilta
vaikeissa vaikeista vaikka vain varmasti varsin varsinkin varten vasen
vasemmalla vasta vastaan vastakkain vastan verran vielä vierekkäin vieressä
vieri viiden viime viimeinen viimeisen viimeksi viisi voi voidaan voimme voin
voisi voit voitte voivat vuoden vuoksi vuosi vuosien vuosina vuotta vähemmän
vähintään vähiten vähän välillä

yhdeksän yhden yhdessä yhteen yhteensä yhteydessä yhteyteen yhtä yhtäälle
yhtäällä yhtäältä yhtään yhä yksi yksin yksittäin yleensä ylemmäs yli ylös
ympäri

älköön älä

""".split())



# encoding: utf8
from __future__ import unicode_literals

from ...symbols import ORTH, LEMMA


_exc = {
    "ม.ค.": [{ORTH: "ม.ค.", LEMMA: "มกราคม"}],
    "ก.พ.": [{ORTH: "ก.พ.", LEMMA: "กุมภาพันธ์"}],
    "มี.ค.": [{ORTH: "มี.ค.", LEMMA: "มีนาคม"}],
    "เม.ย.": [{ORTH: "เม.ย.", LEMMA: "เมษายน"}],
    "พ.ค.": [{ORTH: "พ.ค.", LEMMA: "พฤษภาคม"}],
    "มิ.ย.": [{ORTH: "มิ.ย.", LEMMA: "มิถุนายน"}],
    "ก.ค.": [{ORTH: "ก.ค.", LEMMA: "กรกฎาคม"}],
    "ส.ค.": [{ORTH: "ส.ค.", LEMMA: "สิงหาคม"}],
    "ก.ย.": [{ORTH: "ก.ย.", LEMMA: "กันยายน"}],
    "ต.ค.": [{ORTH: "ต.ค.", LEMMA: "ตุลาคม"}],
    "พ.ย.": [{ORTH: "พ.ย.", LEMMA: "พฤศจิกายน"}],
    "ธ.ค.": [{ORTH: "ธ.ค.", LEMMA: "ธันวาคม"}]
}


TOKENIZER_EXCEPTIONS = _exc



# cython: infer_types=True
# cython: profile=True
cimport numpy as np
import numpy
from cpython.ref cimport PyObject, Py_XDECREF
from thinc.extra.search cimport Beam
from thinc.extra.search import MaxViolation
from thinc.typedefs cimport hash_t, class_t
from thinc.extra.search cimport MaxViolation

from .transition_system cimport TransitionSystem, Transition
from ..gold cimport GoldParse
from .stateclass cimport StateC, StateClass


# These are passed as callbacks to thinc.search.Beam
cdef int _transition_state(void* _dest, void* _src, class_t clas, void* _moves) except -1:
    dest = <StateC*>_dest
    src = <StateC*>_src
    moves = <const Transition*>_moves
    dest.clone(src)
    moves[clas].do(dest, moves[clas].label)
    dest.push_hist(clas)


cdef int _check_final_state(void* _state, void* extra_args) except -1:
    state = <StateC*>_state
    return state.is_final()


cdef hash_t _hash_state(void* _state, void* _) except 0:
    state = <StateC*>_state
    if state.is_final():
        return 1
    else:
        return state.hash()


cdef class ParserBeam(object):
    cdef public TransitionSystem moves
    cdef public object states
    cdef public object golds
    cdef public object beams
    cdef public object dones

    def __init__(self, TransitionSystem moves, states, golds,
                 int width, float density):
        self.moves = moves
        self.states = states
        self.golds = golds
        self.beams = []
        cdef Beam beam
        cdef StateClass state
        cdef StateC* st
        for state in states:
            beam = Beam(self.moves.n_moves, width, density)
            beam.initialize(self.moves.init_beam_state, state.c.length,
                            state.c._sent)
            for i in range(beam.width):
                st = <StateC*>beam.at(i)
                st.offset = state.c.offset
            self.beams.append(beam)
        self.dones = [False] * len(self.beams)

    @property
    def is_done(self):
        return all(b.is_done or self.dones[i]
                   for i, b in enumerate(self.beams))

    def __getitem__(self, i):
        return self.beams[i]

    def __len__(self):
        return len(self.beams)

    def advance(self, scores, follow_gold=False):
        cdef Beam beam
        for i, beam in enumerate(self.beams):
            if beam.is_done or not scores[i].size or self.dones[i]:
                continue
            self._set_scores(beam, scores[i])
            if self.golds is not None:
                self._set_costs(beam, self.golds[i], follow_gold=follow_gold)
            beam.advance(_transition_state, NULL, <void*>self.moves.c)
            beam.check_done(_check_final_state, NULL)
            # This handles the non-monotonic stuff for the parser.
            if beam.is_done and self.golds is not None:
                for j in range(beam.size):
                    state = StateClass.borrow(<StateC*>beam.at(j))
                    if state.is_final():
                        try:
                            if self.moves.is_gold_parse(state, self.golds[i]):
                                beam._states[j].loss = 0.0
                            elif beam._states[j].loss == 0.0:
                                beam._states[j].loss = 1.0
                        except NotImplementedError:
                            break

    def _set_scores(self, Beam beam, float[:, ::1] scores):
        cdef float* c_scores = &scores[0, 0]
        cdef int nr_state = min(scores.shape[0], beam.size)
        cdef int nr_class = scores.shape[1]
        for i in range(nr_state):
            state = <StateC*>beam.at(i)
            if not state.is_final():
                for j in range(nr_class):
                    beam.scores[i][j] = c_scores[i * nr_class + j]
                self.moves.set_valid(beam.is_valid[i], state)
            else:
                for j in range(beam.nr_class):
                    beam.scores[i][j] = 0
                    beam.costs[i][j] = 0

    def _set_costs(self, Beam beam, GoldParse gold, int follow_gold=False):
        for i in range(beam.size):
            state = StateClass.borrow(<StateC*>beam.at(i))
            if not state.is_final():
                self.moves.set_costs(beam.is_valid[i], beam.costs[i],
                                     state, gold)
                if follow_gold:
                    for j in range(beam.nr_class):
                        if beam.costs[i][j] >= 1:
                            beam.is_valid[i][j] = 0


def get_token_ids(states, int n_tokens):
    cdef StateClass state
    cdef np.ndarray ids = numpy.zeros((len(states), n_tokens),
                                      dtype='int32', order='C')
    c_ids = <int*>ids.data
    for i, state in enumerate(states):
        if not state.is_final():
            state.c.set_context_tokens(c_ids, n_tokens)
        else:
            ids[i] = -1
        c_ids += ids.shape[1]
    return ids


nr_update = 0


def update_beam(TransitionSystem moves, int nr_feature, int max_steps,
                states, golds,
                state2vec, vec2scores,
                int width, float density, int hist_feats,
                losses=None, drop=0.):
    global nr_update
    cdef MaxViolation violn
    nr_update += 1
    pbeam = ParserBeam(moves, states, golds,
                       width=width, density=density)
    gbeam = ParserBeam(moves, states, golds,
                       width=width, density=density)
    cdef StateClass state
    beam_maps = []
    backprops = []
    violns = [MaxViolation() for _ in range(len(states))]
    for t in range(max_steps):
        if pbeam.is_done and gbeam.is_done:
            break
        # The beam maps let us find the right row in the flattened scores
        # arrays for each state. States are identified by (example id,
        # history). We keep a different beam map for each step (since we'll
        # have a flat scores array for each step). The beam map will let us
        # take the per-state losses, and compute the gradient for each (step,
        # state, class).
        beam_maps.append({})
        # Gather all states from the two beams in a list. Some stats may occur
        # in both beams. To figure out which beam each state belonged to,
        # we keep two lists of indices, p_indices and g_indices
        states, p_indices, g_indices = get_states(pbeam, gbeam, beam_maps[-1],
                                                  nr_update)
        if not states:
            break
        # Now that we have our flat list of states, feed them through the model
        token_ids = get_token_ids(states, nr_feature)
        vectors, bp_vectors = state2vec.begin_update(token_ids, drop=drop)
        if hist_feats:
            hists = numpy.asarray([st.history[:hist_feats] for st in states],
                                  dtype='i')
            scores, bp_scores = vec2scores.begin_update((vectors, hists),
                                                        drop=drop)
        else:
            scores, bp_scores = vec2scores.begin_update(vectors, drop=drop)

        # Store the callbacks for the backward pass
        backprops.append((token_ids, bp_vectors, bp_scores))

        # Unpack the flat scores into lists for the two beams. The indices arrays
        # tell us which example and state the scores-row refers to.
        p_scores = [numpy.ascontiguousarray(scores[indices], dtype='f')
                    for indices in p_indices]
        g_scores = [numpy.ascontiguousarray(scores[indices], dtype='f')
                    for indices in g_indices]
        # Now advance the states in the beams. The gold beam is contrained to
        # to follow only gold analyses.
        pbeam.advance(p_scores)
        gbeam.advance(g_scores, follow_gold=True)
        # Track the "maximum violation", to use in the update.
        for i, violn in enumerate(violns):
            violn.check_crf(pbeam[i], gbeam[i])
    histories = []
    losses = []
    for violn in violns:
        if violn.p_hist:
            histories.append(violn.p_hist + violn.g_hist)
            losses.append(violn.p_probs + violn.g_probs)
        else:
            histories.append([])
            losses.append([])
    states_d_scores = get_gradient(moves.n_moves, beam_maps, histories, losses)
    beams = list(pbeam.beams) + list(gbeam.beams)
    return states_d_scores, backprops[:len(states_d_scores)], beams


def get_states(pbeams, gbeams, beam_map, nr_update):
    seen = {}
    states = []
    p_indices = []
    g_indices = []
    cdef Beam pbeam, gbeam
    assert len(pbeams) == len(gbeams)
    for eg_id, (pbeam, gbeam) in enumerate(zip(pbeams, gbeams)):
        p_indices.append([])
        g_indices.append([])
        for i in range(pbeam.size):
            state = StateClass.borrow(<StateC*>pbeam.at(i))
            if not state.is_final():
                key = tuple([eg_id] + pbeam.histories[i])
                assert key not in seen, (key, seen)
                seen[key] = len(states)
                p_indices[-1].append(len(states))
                states.append(state)
        beam_map.update(seen)
        for i in range(gbeam.size):
            state = StateClass.borrow(<StateC*>gbeam.at(i))
            if not state.is_final():
                key = tuple([eg_id] + gbeam.histories[i])
                if key in seen:
                    g_indices[-1].append(seen[key])
                else:
                    g_indices[-1].append(len(states))
                    beam_map[key] = len(states)
                    states.append(state)
    p_idx = [numpy.asarray(idx, dtype='i') for idx in p_indices]
    g_idx = [numpy.asarray(idx, dtype='i') for idx in g_indices]
    return states, p_idx, g_idx


def get_gradient(nr_class, beam_maps, histories, losses):
    """The global model assigns a loss to each parse. The beam scores
    are additive, so the same gradient is applied to each action
    in the history. This gives the gradient of a single *action*
    for a beam state -- so we have "the gradient of loss for taking
    action i given history H."

    Histories: Each hitory is a list of actions
    Each candidate has a history
    Each beam has multiple candidates
    Each batch has multiple beams
    So history is list of lists of lists of ints
    """
    nr_step = len(beam_maps)
    grads = []
    nr_step = 0
    for eg_id, hists in enumerate(histories):
        for loss, hist in zip(losses[eg_id], hists):
            if loss != 0.0 and not numpy.isnan(loss):
                nr_step = max(nr_step, len(hist))
    for i in range(nr_step):
        grads.append(numpy.zeros((max(beam_maps[i].values())+1, nr_class),
                                 dtype='f'))
    assert len(histories) == len(losses)
    for eg_id, hists in enumerate(histories):
        for loss, hist in zip(losses[eg_id], hists):
            if loss == 0.0 or numpy.isnan(loss):
                continue
            key = tuple([eg_id])
            # Adjust loss for length
            avg_loss = loss / len(hist)
            loss += avg_loss * (nr_step - len(hist))
            for j, clas in enumerate(hist):
                i = beam_maps[j][key]
                # In step j, at state i action clas
                # resulted in loss
                grads[j][i, clas] += loss
                key = key + tuple([clas])
    return grads



# coding: utf-8
from __future__ import unicode_literals

import pytest
from ....tokens.doc import Doc


@pytest.fixture
def ru_lemmatizer(RU):
    return RU.Defaults.create_lemmatizer()


@pytest.mark.models('ru')
def test_doc_lemmatization(RU):
    doc = Doc(RU.vocab, words=['мама', 'мыла', 'раму'])
    doc[0].tag_ = 'NOUN__Animacy=Anim|Case=Nom|Gender=Fem|Number=Sing'
    doc[1].tag_ = 'VERB__Aspect=Imp|Gender=Fem|Mood=Ind|Number=Sing|Tense=Past|VerbForm=Fin|Voice=Act'
    doc[2].tag_ = 'NOUN__Animacy=Anim|Case=Acc|Gender=Fem|Number=Sing'

    lemmas = [token.lemma_ for token in doc]
    assert lemmas == ['мама', 'мыть', 'рама']


@pytest.mark.models('ru')
@pytest.mark.parametrize('text,lemmas', [('гвоздики', ['гвоздик', 'гвоздика']),
                                         ('люди', ['человек']),
                                         ('реки', ['река']),
                                         ('кольцо', ['кольцо']),
                                         ('пепперони', ['пепперони'])])
def test_ru_lemmatizer_noun_lemmas(ru_lemmatizer, text, lemmas):
    assert sorted(ru_lemmatizer.noun(text)) == lemmas


@pytest.mark.models('ru')
@pytest.mark.parametrize('text,pos,morphology,lemma', [('рой', 'NOUN', None, 'рой'),
                                                       ('рой', 'VERB', None, 'рыть'),
                                                       ('клей', 'NOUN', None, 'клей'),
                                                       ('клей', 'VERB', None, 'клеить'),
                                                       ('три', 'NUM', None, 'три'),
                                                       ('кос', 'NOUN', {'Number': 'Sing'}, 'кос'),
                                                       ('кос', 'NOUN', {'Number': 'Plur'}, 'коса'),
                                                       ('кос', 'ADJ', None, 'косой'),
                                                       ('потом', 'NOUN', None, 'пот'),
                                                       ('потом', 'ADV', None, 'потом')
                                                       ])
def test_ru_lemmatizer_works_with_different_pos_homonyms(ru_lemmatizer, text, pos, morphology, lemma):
    assert ru_lemmatizer(text, pos, morphology) == [lemma]


@pytest.mark.models('ru')
@pytest.mark.parametrize('text,morphology,lemma', [('гвоздики', {'Gender': 'Fem'}, 'гвоздика'),
                                                   ('гвоздики', {'Gender': 'Masc'}, 'гвоздик'),
                                                   ('вина', {'Gender': 'Fem'}, 'вина'),
                                                   ('вина', {'Gender': 'Neut'}, 'вино')
                                                   ])
def test_ru_lemmatizer_works_with_noun_homonyms(ru_lemmatizer, text, morphology, lemma):
    assert ru_lemmatizer.noun(text, morphology) == [lemma]


@pytest.mark.models('ru')
def test_ru_lemmatizer_punct(ru_lemmatizer):
    assert ru_lemmatizer.punct('«') == ['"']
    assert ru_lemmatizer.punct('»') == ['"']


# @pytest.mark.models('ru')
# def test_ru_lemmatizer_lemma_assignment(RU):
#     text = "А роза упала на лапу Азора."
#     doc = RU.make_doc(text)
#     RU.tagger(doc)
#     assert all(t.lemma_ != '' for t in doc)



# coding: utf8
from __future__ import unicode_literals

import pytest


ABBREVIATION_TESTS = [
    ('Hyvää uutta vuotta t. siht. Niemelä!', ['Hyvää', 'uutta', 'vuotta', 't.', 'siht.', 'Niemelä', '!']),
    ('Paino on n. 2.2 kg', ['Paino', 'on', 'n.', '2.2', 'kg'])
]


@pytest.mark.parametrize('text,expected_tokens', ABBREVIATION_TESTS)
def test_tokenizer_handles_testcases(fi_tokenizer, text, expected_tokens):
    tokens = fi_tokenizer(text)
    token_list = [token.text for token in tokens if not token.is_space]
    assert expected_tokens == token_list



# coding: utf8
from __future__ import unicode_literals
from thinc.neural import Model
import pytest
import numpy

from ..._ml import chain, Tok2Vec, doc2feats
from ...vocab import Vocab
from ...pipeline import Tensorizer
from ...syntax.arc_eager import ArcEager
from ...syntax.nn_parser import Parser
from ...tokens.doc import Doc
from ...gold import GoldParse


@pytest.fixture
def vocab():
    return Vocab()


@pytest.fixture
def arc_eager(vocab):
    actions = ArcEager.get_actions(left_labels=['L'], right_labels=['R'])
    return ArcEager(vocab.strings, actions)


@pytest.fixture
def tok2vec():
    return Tok2Vec(8, 100)


@pytest.fixture
def parser(vocab, arc_eager):
    return Parser(vocab, moves=arc_eager, model=None)

@pytest.fixture
def model(arc_eager, tok2vec):
    return Parser.Model(arc_eager.n_moves, token_vector_width=tok2vec.nO,
                        hist_size=0)[0]

@pytest.fixture
def doc(vocab):
    return Doc(vocab, words=['a', 'b', 'c'])

@pytest.fixture
def gold(doc):
    return GoldParse(doc, heads=[1, 1, 1], deps=['L', 'ROOT', 'R'])


def test_can_init_nn_parser(parser):
    assert parser.model is None


def test_build_model(parser):
    parser.model = Parser.Model(parser.moves.n_moves, hist_size=0)[0]
    assert parser.model is not None


def test_predict_doc(parser, tok2vec, model, doc):
    doc.tensor = tok2vec([doc])[0]
    parser.model = model
    parser(doc)


def test_update_doc(parser, model, doc, gold):
    parser.model = model
    def optimize(weights, gradient, key=None):
        weights -= 0.001 * gradient
    parser.update([doc], [gold], sgd=optimize)


def test_predict_doc_beam(parser, model, doc):
    parser.model = model
    parser(doc, beam_width=32, beam_density=0.001)


def test_update_doc_beam(parser, model, doc, gold):
    parser.model = model
    def optimize(weights, gradient, key=None):
        weights -= 0.001 * gradient
    parser.update_beam([doc], [gold], sgd=optimize)





# coding: utf-8
from __future__ import unicode_literals

import pytest

@pytest.mark.parametrize('text',
                         ["ca.", "m.a.o.", "Jan.", "Dec.", "kr.", "jf."])
def test_da_tokenizer_handles_abbr(da_tokenizer, text):
    tokens = da_tokenizer(text)
    assert len(tokens) == 1

@pytest.mark.parametrize('text', ["Jul.", "jul.", "Tor.", "Tors."])
def test_da_tokenizer_handles_ambiguous_abbr(da_tokenizer, text):
    tokens = da_tokenizer(text)
    assert len(tokens) == 2

@pytest.mark.parametrize('text', ["1.", "10.", "31."])
def test_da_tokenizer_handles_dates(da_tokenizer, text):
    tokens = da_tokenizer(text)
    assert len(tokens) == 1

def test_da_tokenizer_handles_exc_in_text(da_tokenizer):
    text = "Det er bl.a. ikke meningen"
    tokens = da_tokenizer(text)
    assert len(tokens) == 5
    assert tokens[2].text == "bl.a."

def test_da_tokenizer_handles_custom_base_exc(da_tokenizer):
    text = "Her er noget du kan kigge i."
    tokens = da_tokenizer(text)
    assert len(tokens) == 8
    assert tokens[6].text == "i"
    assert tokens[7].text == "."

@pytest.mark.parametrize('text,norm',
                         [("akvarium", "akvarie"), ("bedstemoder", "bedstemor")])
def test_da_tokenizer_norm_exceptions(da_tokenizer, text, norm):
    tokens = da_tokenizer(text)
    assert tokens[0].norm_ == norm



# coding: utf8
from __future__ import unicode_literals


STOP_WORDS = set("""
a abban ahhoz ahogy ahol aki akik akkor akár alatt amely amelyek amelyekben
amelyeket amelyet amelynek ami amikor amit amolyan amíg annak arra arról az
azok azon azonban azt aztán azután azzal azért

be belül benne bár

cikk cikkek cikkeket csak

de

e ebben eddig egy egyes egyetlen egyik egyre egyéb egész ehhez ekkor el ellen
elo eloször elott elso elég előtt emilyen ennek erre ez ezek ezen ezt ezzel
ezért

fel felé

ha hanem hiszen hogy hogyan hát

ide igen ill ill. illetve ilyen ilyenkor inkább is ismét ison itt

jobban jó jól

kell kellett keressünk keresztül ki kívül között közül

le legalább legyen lehet lehetett lenne lenni lesz lett

ma maga magát majd meg mellett mely melyek mert mi miatt mikor milyen minden
mindenki mindent mindig mint mintha mit mivel miért mondta most már más másik
még míg

nagy nagyobb nagyon ne nekem neki nem nincs néha néhány nélkül

o oda ok oket olyan ott

pedig persze például

rá

s saját sem semmi sok sokat sokkal stb. szemben szerint szinte számára szét

talán te tehát teljes ti tovább továbbá több túl ugyanis

utolsó után utána

vagy vagyis vagyok valaki valami valamint való van vannak vele vissza viszont
volna volt voltak voltam voltunk

által általában át

én éppen és

így

ön össze

úgy új újabb újra

ő őket
""".split())



from __future__ import unicode_literals
import pytest
from ...lang.en import English
from ...util import load_model


def test_issue1242_empty_strings():
    nlp = English()
    doc = nlp('')
    assert len(doc) == 0
    docs = list(nlp.pipe(['', 'hello']))
    assert len(docs[0]) == 0
    assert len(docs[1]) == 1


@pytest.mark.models('en')
def test_issue1242_empty_strings_en_core_web_sm():
    nlp = load_model('en_core_web_sm')
    doc = nlp('')
    assert len(doc) == 0
    docs = list(nlp.pipe(['', 'hello']))
    assert len(docs[0]) == 0
    assert len(docs[1]) == 1



# coding: utf8
from __future__ import unicode_literals, division, print_function

import plac
from timeit import default_timer as timer
import random
import numpy.random

from ..gold import GoldCorpus
from ..util import prints
from .. import util
from .. import displacy


random.seed(0)
numpy.random.seed(0)


@plac.annotations(
    model=("model name or path", "positional", None, str),
    data_path=("location of JSON-formatted evaluation data", "positional",
               None, str),
    gold_preproc=("use gold preprocessing", "flag", "G", bool),
    gpu_id=("use GPU", "option", "g", int),
    displacy_path=("directory to output rendered parses as HTML", "option",
                   "dp", str),
    displacy_limit=("limit of parses to render as HTML", "option", "dl", int))
def evaluate(model, data_path, gpu_id=-1, gold_preproc=False, displacy_path=None,
             displacy_limit=25):
    """
    Evaluate a model. To render a sample of parses in a HTML file, set an
    output directory as the displacy_path argument.
    """
    if gpu_id >= 0:
        util.use_gpu(gpu_id)
    util.set_env_log(False)
    data_path = util.ensure_path(data_path)
    displacy_path = util.ensure_path(displacy_path)
    if not data_path.exists():
        prints(data_path, title="Evaluation data not found", exits=1)
    if displacy_path and not displacy_path.exists():
        prints(displacy_path, title="Visualization output directory not found",
               exits=1)
    corpus = GoldCorpus(data_path, data_path)
    nlp = util.load_model(model)
    dev_docs = list(corpus.dev_docs(nlp, gold_preproc=gold_preproc))
    begin = timer()
    scorer = nlp.evaluate(dev_docs, verbose=False)
    end = timer()
    nwords = sum(len(doc_gold[0]) for doc_gold in dev_docs)
    print_results(scorer, time=end - begin, words=nwords,
                  wps=nwords / (end - begin))
    if displacy_path:
        docs, golds = zip(*dev_docs)
        render_deps = 'parser' in nlp.meta.get('pipeline', [])
        render_ents = 'ner' in nlp.meta.get('pipeline', [])
        render_parses(docs, displacy_path, model_name=model,
                      limit=displacy_limit, deps=render_deps, ents=render_ents)
        msg = "Generated %s parses as HTML" % displacy_limit
        prints(displacy_path, title=msg)


def render_parses(docs, output_path, model_name='', limit=250, deps=True,
                  ents=True):
    docs[0].user_data['title'] = model_name
    if ents:
        with (output_path / 'entities.html').open('w') as file_:
            html = displacy.render(docs[:limit], style='ent', page=True)
            file_.write(html)
    if deps:
        with (output_path / 'parses.html').open('w') as file_:
            html = displacy.render(docs[:limit], style='dep', page=True,
                                   options={'compact': True})
            file_.write(html)


def print_progress(itn, losses, dev_scores, wps=0.0):
    scores = {}
    for col in ['dep_loss', 'tag_loss', 'uas', 'tags_acc', 'token_acc',
                'ents_p', 'ents_r', 'ents_f', 'wps']:
        scores[col] = 0.0
    scores['dep_loss'] = losses.get('parser', 0.0)
    scores['ner_loss'] = losses.get('ner', 0.0)
    scores['tag_loss'] = losses.get('tagger', 0.0)
    scores.update(dev_scores)
    scores['wps'] = wps
    tpl = '\t'.join((
        '{:d}',
        '{dep_loss:.3f}',
        '{ner_loss:.3f}',
        '{uas:.3f}',
        '{ents_p:.3f}',
        '{ents_r:.3f}',
        '{ents_f:.3f}',
        '{tags_acc:.3f}',
        '{token_acc:.3f}',
        '{wps:.1f}'))
    print(tpl.format(itn, **scores))


def print_results(scorer, time, words, wps):
    results = {
        'Time': '%.2f s' % time,
        'Words': words,
        'Words/s': '%.0f' % wps,
        'TOK': '%.2f' % scorer.token_acc,
        'POS': '%.2f' % scorer.tags_acc,
        'UAS': '%.2f' % scorer.uas,
        'LAS': '%.2f' % scorer.las,
        'NER P': '%.2f' % scorer.ents_p,
        'NER R': '%.2f' % scorer.ents_r,
        'NER F': '%.2f' % scorer.ents_f}
    util.print_table(results, title="Results")



# coding: utf-8
from __future__ import unicode_literals
import re

from ...matcher import Matcher

import pytest

pattern1	= [{'ORTH':'A','OP':'1'},{'ORTH':'A','OP':'*'}]
pattern2	= [{'ORTH':'A','OP':'*'},{'ORTH':'A','OP':'1'}]
pattern3	= [{'ORTH':'A','OP':'1'},{'ORTH':'A','OP':'1'}]
pattern4	= [{'ORTH':'B','OP':'1'},{'ORTH':'A','OP':'*'},{'ORTH':'B','OP':'1'}]
pattern5 	= [{'ORTH':'B','OP':'*'},{'ORTH':'A','OP':'*'},{'ORTH':'B','OP':'1'}]

re_pattern1	= 'AA*'
re_pattern2 = 'A*A'
re_pattern3	= 'AA'
re_pattern4	= 'BA*B'
re_pattern5	= 'B*A*B'

@pytest.fixture
def text():
	return "(ABBAAAAAB)."

@pytest.fixture
def doc(en_tokenizer,text):
    doc = en_tokenizer(' '.join(text))
    return doc

@pytest.mark.parametrize('pattern,re_pattern',[
	(pattern1,re_pattern1),
	(pattern2,re_pattern2),
	(pattern3,re_pattern3),
	(pattern4,re_pattern4),
	(pattern5,re_pattern5)])
def test_greedy_matching(doc,text,pattern,re_pattern):
	"""
	Test that the greedy matching behavior of the * op
	is consistant with other re implementations
	"""
	matcher = Matcher(doc.vocab)
	matcher.add(re_pattern,None,pattern)
	matches = matcher(doc)
	re_matches = [m.span() for m in re.finditer(re_pattern,text)]
	for match,re_match in zip(matches,re_matches):
		assert match[1:]==re_match

@pytest.mark.parametrize('pattern,re_pattern',[
	(pattern1,re_pattern1),
	(pattern2,re_pattern2),
	(pattern3,re_pattern3),
	(pattern4,re_pattern4),
	(pattern5,re_pattern5)])
def test_match_consuming(doc,text,pattern,re_pattern):
	"""
	Test that matcher.__call__ consumes tokens on a match
	similar to re.findall
	"""
	matcher = Matcher(doc.vocab)
	matcher.add(re_pattern,None,pattern)
	matches = matcher(doc)
	re_matches = [m.span() for m in re.finditer(re_pattern,text)]
	assert len(matches)==len(re_matches)


from __future__ import unicode_literals
import pytest
import spacy


def ss(tt):
    for i in range(len(tt)-1):
        for j in range(i+1, len(tt)):
            tt[i:j].root


@pytest.mark.models('en')
def test_access_parse_for_merged():
    nlp = spacy.load('en_core_web_sm')
    t_t = nlp.tokenizer("Highly rated - I'll definitely")
    nlp.tagger(t_t)
    nlp.parser(t_t)
    nlp.parser(t_t)
    ss(t_t)




from __future__ import unicode_literals
import pytest

from ...language import Language

def test_issue1380_empty_string():
    nlp = Language()
    doc = nlp('')
    assert len(doc) == 0

@pytest.mark.models('en')
def test_issue1380_en(EN):
    doc = EN('')
    assert len(doc) == 0



# coding: utf-8
from __future__ import unicode_literals

from ...strings import StringStore

import pytest


def test_string_hash(stringstore):
    '''Test that string hashing is stable across platforms'''
    ss = stringstore
    assert ss.add('apple') == 8566208034543834098
    heart = '\U0001f499'
    print(heart)
    h = ss.add(heart)
    assert h == 11841826740069053588
 

def test_stringstore_from_api_docs(stringstore):
    apple_hash = stringstore.add('apple')
    assert apple_hash == 8566208034543834098
    assert stringstore[apple_hash] == u'apple'

    assert u'apple' in stringstore
    assert u'cherry' not in stringstore

    orange_hash = stringstore.add('orange')
    all_strings = [s for s in stringstore]
    assert all_strings == [u'apple', u'orange']

    banana_hash = stringstore.add('banana')
    assert len(stringstore) == 3
    assert banana_hash == 2525716904149915114
    assert stringstore[banana_hash] == u'banana'
    assert stringstore[u'banana'] == banana_hash


@pytest.mark.parametrize('text1,text2,text3', [(b'Hello', b'goodbye', b'hello')])
def test_stringstore_save_bytes(stringstore, text1, text2, text3):
    key = stringstore.add(text1)
    assert stringstore[text1] == key
    assert stringstore[text2] != key
    assert stringstore[text3] != key


@pytest.mark.parametrize('text1,text2,text3', [('Hello', 'goodbye', 'hello')])
def test_stringstore_save_unicode(stringstore, text1, text2, text3):
    key = stringstore.add(text1)
    assert stringstore[text1] == key
    assert stringstore[text2] != key
    assert stringstore[text3] != key


@pytest.mark.parametrize('text', [b'A'])
def test_stringstore_retrieve_id(stringstore, text):
    key = stringstore.add(text)
    assert len(stringstore) == 1
    assert stringstore[key] == text.decode('utf8')
    with pytest.raises(KeyError):
        stringstore[20000]


@pytest.mark.parametrize('text1,text2', [(b'0123456789', b'A')])
def test_stringstore_med_string(stringstore, text1, text2):
    store = stringstore.add(text1)
    assert stringstore[store] == text1.decode('utf8')
    dummy = stringstore.add(text2)
    assert stringstore[text1] == store


def test_stringstore_long_string(stringstore):
    text = "INFORMATIVE](http://www.google.com/search?as_q=RedditMonkey&amp;hl=en&amp;num=50&amp;btnG=Google+Search&amp;as_epq=&amp;as_oq=&amp;as_eq=&amp;lr=&amp;as_ft=i&amp;as_filetype=&amp;as_qdr=all&amp;as_nlo=&amp;as_nhi=&amp;as_occt=any&amp;as_dt=i&amp;as_sitesearch=&amp;as_rights=&amp;safe=off"
    store = stringstore.add(text)
    assert stringstore[store] == text


@pytest.mark.parametrize('factor', [254, 255, 256])
def test_stringstore_multiply(stringstore, factor):
    text = 'a' * factor
    store = stringstore.add(text)
    assert stringstore[store] == text


def test_stringstore_massive_strings(stringstore):
    text = 'a' * 511
    store = stringstore.add(text)
    assert stringstore[store] == text
    text2 = 'z' * 512
    store = stringstore.add(text2)
    assert stringstore[store] == text2
    text3 = '1' * 513
    store = stringstore.add(text3)
    assert stringstore[store] == text3


@pytest.mark.parametrize('text', ["qqqqq"])
def test_stringstore_to_bytes(stringstore, text):
    store = stringstore.add(text)
    serialized = stringstore.to_bytes()
    new_stringstore = StringStore().from_bytes(serialized)
    assert new_stringstore[store] == text
